-------------------------------------------------------------
-- MSS copyright 2012-2013
--	Filename:  CLEAN_CLK.VHD
-- Author: Alain Zarembowitch / MSS
--	Version: 1
--	Date last modified: 7/1/13
-- Inheritance: 	N/A
--
-- description:  'cleans' an input clock by making output clock cycles 
-- as regularly spaced as possible, with a duty cycle as close to 50% as possible.
--
-- Rev 1  7/1/13 AZ
-- bug: output clock could be slightly slower than input clock, leading to underflow conditions
---------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
Library UNISIM;
use UNISIM.vcomponents.all;

entity CLEAN_CLK is
    port ( 
		--GLOBAL CLOCKS, RESET
		CLK : in std_logic;				-- synchronous clock
		SYNC_RESET: in std_logic;

		CLK_PULSE_IN: in std_logic;
			-- input clock (1-CLK-wide pulses)
		CLK_50DUTY_OUT: out std_logic;
			-- 'cleaned' output clock with regularly spaced cycles and duty cycle as close to 50% as possible
		CLK_PULSE_OUT: out std_logic
			-- 'cleaned' output clock as short 1 CLK-wide pulses, as regularly spaced as possible
			);
end entity;

architecture behavioral of CLEAN_CLK is
--------------------------------------------------------
--      COMPONENTS
--------------------------------------------------------
	COMPONENT MEASURE_INTERVAL
	PORT(
		CLK : IN std_logic;
		SYNC_RESET : IN std_logic;
		PULSE_IN : IN std_logic;          
		PERIOD : OUT std_logic_vector(15 downto 0);
		PERIOD_SAMPLE_CLK : OUT std_logic
		);
	END COMPONENT;
--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------
signal PERIOD: std_logic_vector(15 downto 0) := (others => '0');
signal PERIOD_SAMPLE_CLK: std_logic := '0';
signal PERIOD_CNTR: std_logic_vector(15 downto 0) := (others => '0');
signal DIFF: std_logic_vector(15 downto 0) := (others => '0');
signal BLOCK_OUTPUT: std_logic := '0';

--------------------------------------------------------
--      IMPLEMENTATION
--------------------------------------------------------
begin

-- measure the average clock period
MEASURE_INTERVAL_001: MEASURE_INTERVAL PORT MAP(
	CLK => CLK,
	SYNC_RESET => SYNC_RESET,
	PULSE_IN => CLK_PULSE_IN,
	PERIOD => PERIOD,
	PERIOD_SAMPLE_CLK => PERIOD_SAMPLE_CLK	-- measurement is ready
);

-- regenerate cleaner clock
CLK_GEN_001: process(CLK)
begin
	if rising_edge(CLK) then
		-- modulo-PERIOD counter
		if(PERIOD_CNTR = 0) or (PERIOD_SAMPLE_CLK = '1') then
			-- re-arm the counter
			PERIOD_CNTR <= PERIOD -1;	
			-- The output clock is a little bit faster as PERIOD is truncated. 
			-- The safety valve below will prevent the output clock from ever exceeding the input clock.
		else
			PERIOD_CNTR <= PERIOD_CNTR -1;
		end if;
	end if;
end process;

-- Safety valve to prevent the output clock from ever exceeding the input clock.
-- Track the difference between input and output cycles
SAFETY_001:  process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			DIFF <= (others => '0');
		elsif(CLK_PULSE_IN = '1') and (PERIOD_CNTR = 0) and (BLOCK_OUTPUT = '0') then
			-- 1 clk in, 1 clk out. No change in the difference
		elsif(CLK_PULSE_IN = '1') then
			-- 1 clk in, no clk out.
			DIFF <= DIFF + 1;
		elsif(PERIOD_CNTR = 0) and (BLOCK_OUTPUT = '0') then
			-- no clk in, 1 clk out
			DIFF <= DIFF - 1;
		end if;
	end if;
end process;

-- block periodic output CLK if it would make the difference negative (i.e. more output clks than input clks).
BLOCK_OUTPUT <= '1' when (DIFF = 0) or (DIFF(15) = '1') else '0';

CLK_GEN_002: process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			CLK_50DUTY_OUT <= '0';
		elsif(PERIOD_CNTR = 0) and (BLOCK_OUTPUT = '0') then
			-- new period
			CLK_50DUTY_OUT <= '1';
		elsif(PERIOD_CNTR(14 downto 0) = PERIOD(15 downto 1)) then
			-- half-way point
			CLK_50DUTY_OUT <= '0';
		end if;
	end if;
end process;

CLK_GEN_003: process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			CLK_PULSE_OUT <= '0';
		elsif(PERIOD_CNTR = 0) and (BLOCK_OUTPUT = '0') then
			-- new period
			CLK_PULSE_OUT <= '1';
		else
			CLK_PULSE_OUT <= '0';
		end if;
	end if;
end process;

end architecture;
