-------------------------------------------------------------
-- MSS copyright 2011-2013
--	Filename:  HIGH_SPEED_NCO.VHD
-- Authors: 
--		Alain Zarembowitch / MSS, 
--	Version: 1
--	Date last modified: 6/11/13
-- Inheritance: 	SYMBOL_CLKGEN.vhd 5/10/12
--
-- description: Generate a fully programmable clocks 
-- A fixed PLL generates a fixed frequency CLKPLL(234 MHz clock from a 26 MHz frequency reference, 230 MHz from a 30 MHz freq ref).
-- A follow-on DDS generates a clock with fine-programmable frequency 
-- For maximum speed, the DDS is split into two parts: a 10-bit DDS with a step size alternating
-- between N and N+1 (N represents the upper 10 frequency bits), statistically reflecting 
-- the lower 10 frequency bits. 
-- This code is optimized for speed.
--
-- Rev2 6/11/13 AZ
-- added generic flexibility.
----------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;
library unisim;
use unisim.vcomponents.all;

entity HIGH_SPEED_NCO is
generic (
	REFCLK_FREQUENCY: integer := 26
		-- Reference clock frequency in MHz (only 26,30 supported within)
);
port
 (
  REFCLK: in     std_logic;
	-- input reference clock global. (PLL valid input range 19 - 375 MHz)
  NCO_FREQ: in std_logic_vector(19 downto 0);
	-- DDS frequencies. Expressed as f(in MHz)/225.3333*2^20 or f(in MHz)/220*2^20
	-- The frequencies can be changed on the fly.
  CLK_OUTG          : out    std_logic;
	-- Clock out ports (after BUFGs)
  LOCKED_PLL            : out    std_logic
	-- pll lock status 
 );
end entity;

architecture Behavioral of HIGH_SPEED_NCO is
--------------------------------------------------------
--      COMPONENTS
--------------------------------------------------------
--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------
signal CLKFB1: std_logic;
signal CLKPLL: std_logic;
signal CLKPLLg: std_logic;
signal NCO_ACC: std_logic_vector(9 downto 0) := (others => '0');
signal CLK_FREQA: std_logic_vector(9 downto 0) := (others => '0');
signal CLK_FREQB: std_logic_vector(9 downto 0) := (others => '0');
signal CNTR: std_logic_vector(9 downto 0) := (others => '0');
signal CNTR_SHUFFLED: std_logic_vector(9 downto 0) := (others => '0');
signal NPLUSONESEL: std_logic := '0';
signal NCO_ACC_MSB_DR: std_logic := '0';
signal NCO_ACC_MSB_DR_D: std_logic := '0';
signal NCO_ACC_MSB_DF: std_logic := '0';
signal NCO_ACC_MSB_DF_D: std_logic := '0';
signal NCO_ACC_2MSBs_D: std_logic_vector(1 downto 0) := (others => '0');
signal NCO_ACC_2MSBs_D2: std_logic_vector(1 downto 0) := (others => '0');
signal NCO_ACC_2MSBs_DIFF: std_logic_vector(1 downto 0) := (others => '0');
signal NCO_ACC_2MSBs_DIFF_D: std_logic_vector(1 downto 0) := (others => '0');
signal EDGE_SELECT0: std_logic := '0';
signal EDGE_SELECT1: std_logic := '0';
signal EDGE_SELECT: std_logic := '0';
signal DEBUG_CNTRA: std_logic_vector(15 downto 0) := (others => '0'); 
signal DEBUG_CNTRB: std_logic_vector(15 downto 0) := (others => '0'); 
signal DEBUG_CNTRC: std_logic_vector(15 downto 0) := (others => '0'); 
signal CLK_OUT: std_logic := '0';
constant SIMULATION: std_logic := '0';	-- enable/disable file log
--------------------------------------------------------
--      IMPLEMENTATION
--------------------------------------------------------
begin

PLL_26MHZIN: if(REFCLK_FREQUENCY = 26) generate
	-- first PLL generates a fixed 225.3333 MHz PLL frequ from the 26 MHz reference
	PLL_BASE_001 : PLL_BASE
	generic map
	(BANDWIDTH            => "LOW",	-- fixed PLL, prefer low jitter to fast lock
	 CLK_FEEDBACK         => "CLKFBOUT",
	 COMPENSATION         => "INTERNAL",
	 DIVCLK_DIVIDE        => 1,	
	 CLKFBOUT_MULT        => 26,	-- 676 MHz VCO / 26 = 26 MHz input reference freq.
	 CLKFBOUT_PHASE       => 0.000,
	 CLKOUT0_DIVIDE       => 3,	-- 225.3333 MHz output
	 CLKOUT0_PHASE        => 0.0,
	 CLKOUT0_DUTY_CYCLE   => 0.500,
	 CLKIN_PERIOD         => 38.461538461538461538461538461538,
	 REF_JITTER           => 0.010)
	port map
	 -- Output clocks
	(CLKFBOUT            => CLKFB1,
	 CLKOUT0             => CLKPLL,
	 -- Status and control signals
	 LOCKED              => LOCKED_PLL,
	 RST                 => '0',
	 -- Input clock control
	 CLKFBIN             => CLKFB1,
	 CLKIN               => REFCLK
	 );
end generate;
PLL_30MHZIN: if(REFCLK_FREQUENCY = 30) generate
	-- first PLL generates a fixed 220 MHz from the 30 MHz reference
	PLL_BASE_001 : PLL_BASE
	generic map
	(BANDWIDTH            => "LOW",	-- fixed PLL, prefer low jitter to fast lock
	 CLK_FEEDBACK         => "CLKFBOUT",
	 COMPENSATION         => "INTERNAL",
	 DIVCLK_DIVIDE        => 1,	
	 CLKFBOUT_MULT        => 22,	-- 660 MHz VCO / 22 = 30 MHz input reference freq.
	 CLKFBOUT_PHASE       => 0.000,
	 CLKOUT0_DIVIDE       => 3,	-- 220 MHz output
	 CLKOUT0_PHASE        => 0.0,
	 CLKOUT0_DUTY_CYCLE   => 0.500,
	 CLKIN_PERIOD         => 33.333333333333333333333333333333,
	 REF_JITTER           => 0.010)
	port map
	 -- Output clocks
	(CLKFBOUT            => CLKFB1,
	 CLKOUT0             => CLKPLL,
	 -- Status and control signals
	 LOCKED              => LOCKED_PLL,
	 RST                 => '0',
	 -- Input clock control
	 CLKFBIN             => CLKFB1,
	 CLKIN               => REFCLK
	 );
end generate;

-- make CLK0 global
BUFG_001: BUFG port map(I => CLKPLL, O => CLKPLLg);

-- TX DDS to generate the tx symbol rate with fine tuning
NCO_ACC_001A: process(CLKPLLg)
begin
	if rising_edge(CLKPLLg) then
		NCO_ACC <= NCO_ACC + CLK_FREQB;
		
		NCO_ACC_2MSBs_D <= NCO_ACC(9 downto 8);
		NCO_ACC_2MSBs_D2 <=NCO_ACC_2MSBs_D;
		-- compute difference 2 most-significant bits
		NCO_ACC_2MSBs_DIFF <= NCO_ACC(9 downto 8) - NCO_ACC_2MSBs_D;
		NCO_ACC_2MSBs_DIFF_D <= NCO_ACC_2MSBs_DIFF;

		NCO_ACC_MSB_DR <= NCO_ACC_2MSBs_D(1);
		NCO_ACC_MSB_DR_D <= NCO_ACC_MSB_DR;
		
	end if;
end process;

-- convoluted logic to avoid very short clock pulses (i.e. less than a CLKPLLg clock period)
-- Simulate to see how it works.
NCO_ACC_001B: process(CLKPLLg)
begin
	if falling_edge(CLKPLLg) then
		NCO_ACC_MSB_DF <= NCO_ACC_2MSBs_D(1);	-- 1/2 a clock period later
		NCO_ACC_MSB_DF_D <= NCO_ACC_MSB_DF;
	end if;
end process;

EDGE_SELECT1 <= '1' when (NCO_ACC_2MSBs_DIFF_D = 2) and (NCO_ACC_2MSBs_D2(0) = '1') else '0';
EDGE_SELECT0 <= '1' when (NCO_ACC_2MSBs_DIFF = 2) and (NCO_ACC_2MSBs_D(0) = '0') else '0';

EDGE_SELECT <= EDGE_SELECT1 or EDGE_SELECT0;

-- select rising edge or falling edge to reduce peak-to-peak jitter by half
BUFGMUX_001 : BUFGMUX
generic map (
	CLK_SEL_TYPE => "ASYNC"  
)
port map (
	O => CLK_OUTG,   -- 1-bit output. Synthesized clock
	I0 => NCO_ACC_MSB_DR_D, 
	I1 => NCO_ACC_MSB_DF_D, 
	S => EDGE_SELECT    
);

NCO_ACC_002: process(CLKPLLg)
begin
	if rising_edge(CLKPLLg) then
		-- fractional-N DDS: statistically alternate DDS step value between N and N+1 
		if(NPLUSONESEL = '0') then
			CLK_FREQB <= CLK_FREQA;
		else
			CLK_FREQB <= CLK_FREQA + 1;
		end if;
	end if;
end process;

-- make it a fractional-N DDS (it would be easier to do a 20-bit accumulator, but slower)
FRACT_N_001: process(CLKPLLg)
begin
	if rising_edge(CLKPLLg) then
		CNTR <= CNTR + 1;

		-- shuffle the counter bits to randomize the fractional-N DDS
		CNTR_SHUFFLED(9) <= CNTR(0);
		CNTR_SHUFFLED(8) <= CNTR(1);
		CNTR_SHUFFLED(7) <= CNTR(2);
		CNTR_SHUFFLED(6) <= CNTR(3);
		CNTR_SHUFFLED(5) <= CNTR(4);
		CNTR_SHUFFLED(4) <= CNTR(5);
		CNTR_SHUFFLED(3) <= CNTR(6);
		CNTR_SHUFFLED(2) <= CNTR(7);
		CNTR_SHUFFLED(1) <= CNTR(8);
		CNTR_SHUFFLED(0) <= CNTR(9);

		if(CNTR_SHUFFLED > NCO_FREQ(9 downto 0)) then -- lower 10 frequency bits
			NPLUSONESEL <= '0';
		else
			NPLUSONESEL <= '1';
		end if;
	end if;
end process;


-- just reclock for better timing
NCO_ACC_GEN_003: process(CLKPLLg)
begin
	if rising_edge(CLKPLLg) then
		-- no phase correction. just the nominal phase increment.
		CLK_FREQA <= NCO_FREQ(19 downto 10);
	end if;
end process;

--// DEBUG -------------------
DEBUG_001: if(SIMULATION = '1') generate
	-- start of debug
	-- debugging. verify frequencies during simulation
	CNTRA_001: process(CLKPLLg)
	begin
		if rising_edge(CLKPLLg) then
			DEBUG_CNTRA <= DEBUG_CNTRA + 1;
		end if;
	end process;

	CNTRB_001: process(REFCLK)
	begin
		if rising_edge(REFCLK) then
			DEBUG_CNTRB <= DEBUG_CNTRB + 1;
		end if;
	end process;

	BUFGMUX_002 : BUFGMUX
	generic map (
		CLK_SEL_TYPE => "ASYNC"  
	)
	port map (
		O => CLK_OUT,   -- 1-bit output. Synthesized clock
		I0 => NCO_ACC_MSB_DR_D, 
		I1 => NCO_ACC_MSB_DF_D, 
		S => EDGE_SELECT    
	);

	CNTRC_001: process(CLK_OUT)
	begin
		if rising_edge(CLK_OUT) then
			DEBUG_CNTRC <= DEBUG_CNTRC + 1;
		end if;
	end process;
end generate;

end Behavioral;
