-------------------------------------------------------------
--	Filename:  SPI.VHD
-- Author: Alain Zarembowitch / MSS
--	Version: 1
--	Date last modified: 2/10/04
-- Inheritance: 	none
--
-- Description: Serial port interface to read/write registers 
-- from/to AD9860/2 analog front-end IC
-- Supports 8-bit read/write.
-- Latency between SPI_WRITE_REQ and SPI_BUSY is 0 CLK
-- Latency between SPI_READ_REQ and SPI_BUSY is 0 CLK
---------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity SPI is port ( 

	--// Clocks Resets
   CLK : in std_logic;				-- Master clock for this FPGA
	CLK_DIV: in std_logic;			
		-- slow down master clock as maximum SPI speed is 16 MHz
		-- All processes within this module act only when CLK_DIV = '1'.
	ASYNC_RESET: in std_logic;		-- Asynchronous reset active high
		
	--// control
	ADDR: in std_logic_vector(5 downto 0);
		-- This information can change immediately after the request is made
	DATA_OUT: in std_logic_vector(7 downto 0);
		-- byte to be written to specified register
		-- This information can change immediately after the request is made
	WRITE_REQ: in std_logic;
		-- 1 CLK_DIV pulse indicating that the DATA_OUT is to be written out
		-- to the specified register.
	READ_REQ: in std_logic;
		-- 1 CLK_DIV pulse indicating a read request
	DATA_IN: out std_logic_vector(7 downto 0);
		-- byte read from the specified register
	DATA_IN_CLK: out std_logic;
		-- 1 CLK pulse indicating that DATA_IN is valid.
	BUSY: out std_logic;
		-- '0' to indicate that the SPI interface is available. Goes to '1'
		-- immediately after WRITE_REQ and READ_REQ until command is complete.

  --// SPI interface
  SEN: out std_logic;
  SCLK: out std_logic;
  SDIO: out std_logic;
  SDO: in std_logic

);
end entity;

architecture behavioral of SPI is
--------------------------------------------------------
--      COMPONENTS
--------------------------------------------------------
--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------
signal CLK_COUNTER: std_logic_vector(5 downto 0);
signal CLK_COUNTER_IS_ZERO: std_logic;
signal RD_WRN: std_logic;
signal ADDR_D: std_logic_vector(5 downto 0);
signal DATA_OUT_D: std_logic_vector(7 downto 0);
signal DATA_IN_LOCAL: std_logic_vector(7 downto 0);

--------------------------------------------------------
--      IMPLEMENTATION
--------------------------------------------------------

begin

--// The SPI waveform consists of 16 clock cycles, or 34 phases
-- Idle state when CLK_COUNTER = 0.
-- Generate SCLK
-- Important: reclock SCLK to avoid glitches on an edge-trigger signal.
CLK_COUNTER_GEN_001: process(ASYNC_RESET, CLK)
begin
	if(ASYNC_RESET = '1') then
		CLK_COUNTER_IS_ZERO <= '1';
		SCLK <= '1';
	elsif rising_edge(CLK) then
		if(CLK_DIV = '1') then	-- slow down SPI to less than 16 MHz
			if(CLK_COUNTER = 0) and ((WRITE_REQ = '1') or (READ_REQ = '1')) then
				-- Commands are ignored until the current command is completed
			 	-- start counting
				CLK_COUNTER <= "000001";
				CLK_COUNTER_IS_ZERO <= '0';	
					-- for timing purposes. Keep track of whether the CLK_COUNTER is zero
				SCLK <= '1';
			elsif(CLK_COUNTER = 34) then
				CLK_COUNTER <= (others => '0');
				CLK_COUNTER_IS_ZERO <= '1';
				SCLK <= '1';
		  	elsif(CLK_COUNTER /= 0) then
				-- once the cycle is started, finish it.
				CLK_COUNTER <= CLK_COUNTER + 1;
				CLK_COUNTER_IS_ZERO <= '0';
				SCLK <= not(CLK_COUNTER(0));
			end if;
		end if;
	end if;
end process;

-- unavailable for another read/write request until end of cycle
BUSY <= '1' when ((WRITE_REQ = '1') or (READ_REQ = '1') or (CLK_COUNTER_IS_ZERO = '0')) else '0';


-- Generate SEN
SEN_GEN_001: process(ASYNC_RESET, CLK)
begin
	if(ASYNC_RESET = '1') then
		SEN <= '1';
	elsif rising_edge(CLK) then
		if(CLK_DIV = '1') then	-- slow down SPI to less than 16 MHz
			if(CLK_COUNTER = 1) then
				SEN <= '0';
			elsif(CLK_COUNTER = 33) then
				SEN <= '1';
			end if;
		end if;
	end if;
end process;


-- Remember the operation type, the address, the write data
-- This information can change immediately after the request is made
OP_TYPE_001: process(ASYNC_RESET, CLK)
begin
	if(ASYNC_RESET = '1') then
		RD_WRN <= '1';
	elsif rising_edge(CLK) then
		if(CLK_DIV = '1') then	-- slow down SPI to less than 16 MHz
			if(CLK_COUNTER = 0) then
				if(WRITE_REQ = '1') then
					RD_WRN <= '0';	-- serial write starting
					ADDR_D <= ADDR;
					DATA_OUT_D <= DATA_OUT;
				elsif(READ_REQ = '1') then
					RD_WRN <= '1';	-- serial read 
					ADDR_D <= ADDR;
				end if;
			end if;
		end if;
	end if;
end process;


-- Generate SDIO.
SDIO_GEN_001: process(CLK)
begin
	if rising_edge(CLK) then
		if(CLK_DIV = '1') then	-- slow down SPI to less than 16 MHz
			case(CLK_COUNTER) is
				when "000001" => SDIO <= RD_WRN;	
				when "000011" => SDIO <= '0'; 	
				when "000101" => SDIO <= ADDR_D(5);	
				when "000111" => SDIO <= ADDR_D(4);	
				when "001001" => SDIO <= ADDR_D(3);	
				when "001011" => SDIO <= ADDR_D(2);	
				when "001101" => SDIO <= ADDR_D(1);	
				when "001111" => SDIO <= ADDR_D(0);	
				when "010001" => SDIO <= DATA_OUT_D(7) and (not RD_WRN);
				when "010011" => SDIO <= DATA_OUT_D(6) and (not RD_WRN);
				when "010101" => SDIO <= DATA_OUT_D(5) and (not RD_WRN);
				when "010111" => SDIO <= DATA_OUT_D(4) and (not RD_WRN);
				when "011001" => SDIO <= DATA_OUT_D(3) and (not RD_WRN);
				when "011011" => SDIO <= DATA_OUT_D(2) and (not RD_WRN);
				when "011101" => SDIO <= DATA_OUT_D(1) and (not RD_WRN);
				when "011111" => SDIO <= DATA_OUT_D(0) and (not RD_WRN);
				when others => null;
			end case;
		end if;
	end if;
end process;

-- Read serial data in
SDO_READ_001: process(CLK)
begin
	if rising_edge(CLK) then
		if(CLK_DIV = '1') then	-- slow down SPI to less than 16 MHz
			if(RD_WRN = '1') then
				case(CLK_COUNTER) is
					when "010010" => DATA_IN_LOCAL(7) <= SDO;	
					when "010100" => DATA_IN_LOCAL(6) <= SDO;	
					when "010110" => DATA_IN_LOCAL(5) <= SDO;	
					when "011000" => DATA_IN_LOCAL(4) <= SDO;	
					when "011010" => DATA_IN_LOCAL(3) <= SDO;	
					when "011100" => DATA_IN_LOCAL(2) <= SDO;	
					when "011110" => DATA_IN_LOCAL(1) <= SDO;	
					when "100000" => DATA_IN_LOCAL(0) <= SDO;	
					when others => null;
				end case;
			end if;
		end if;
	end if;
end process;

-- At clock 33, reading is complete. Latch read data out.
SDO_READ_002: process(CLK)
begin
	if rising_edge(CLK) then
		if(CLK_DIV = '1') then	-- slow down SPI to less than 16 MHz
			if(RD_WRN = '1') and (CLK_COUNTER = 33) then
				DATA_IN <= DATA_IN_LOCAL;
				DATA_IN_CLK <= '1';
			else
				DATA_IN_CLK <= '0';
			end if;
		else
			DATA_IN_CLK <= '0';
		end if;
	end if;
end process;

end behavioral;