-------------------------------------------------------------
-- MSS copyright 2012-2013
-- Filename:  COM1700.VHD
-- Authors: 
--		Alain Zarembowitch / MSS
-- Version: Rev 0
-- Last modified: 8/4/13
-- Inheritance: 	n/a
--
-- description:  Top level for COM-1700 hardware platform development template.
-- The high-speed USB phy is configured for two virtual channels: one reserved for monitoring and control, 
-- the second can be used by the user to transfer two-way high-speed data streams.
-- Likewise, the LAN is configured for one TCP-IP channel, reserved for monitoring and control (port 1028).
-- User must define the following LAN parameters:
-- IP address, IP Gateway and IP subnet mask
---------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use work.com5402pkg.all;	-- defines global types, number of TCP streams, etc
Library UNISIM;
use UNISIM.vcomponents.all;

entity COM1700 is
    Port ( 
		--GLOBAL CLOCKS
		CLKREF: in std_logic;
			-- higher-stability frequency reference either 26 MHz TCXO or 10x3 MHz external clock through J5 (third harmonic).
		CLKREF25: in std_logic;
			-- 25 MHz frequency clock from LAN PHY. Unused other than for required pulldown(?)
			
		--// Right connector: 98 pin PCIe female connector.   
		-- Pins B5,B20,B31,B42 reserved for Ground
		-- Pins A49/B49 reserved for ComBlock Monitoring and Control.
		RIGHT_CONNECTOR_A: inout std_logic_vector(48 downto 1);
		RIGHT_CONNECTOR_B: inout std_logic_vector(48 downto 1);
		
		--// ADCs
		ADC_CLKP: out std_logic;
		ADC_CLKN: out std_logic;
			-- LVDS differential clock to ADCs
		ADC1_DCO: in std_logic;
		ADC2_DCO: in std_logic;
		ADC1_D: in std_logic_vector(9 downto 0);
		ADC2_D: in std_logic_vector(9 downto 0);
			-- ADC samples, offset binary format. Read at rising edge of ADCx_DCO
		ADC_SPI: inout std_logic_vector(4 downto 1);
		
		--// DACs
		DAC_CLKP: out std_logic;
		DAC_CLKN: out std_logic;
		DAC1_D: out std_logic_vector(11 downto 0);
		DAC2_D: out std_logic_vector(11 downto 0);
		DAC_SPI: inout std_logic_vector(3 downto 1);	-- DAC controls. common to both DACs
		DAC1_SPI_CSB: out std_logic;	
		DAC2_SPI_CSB: out std_logic;
		
		--// D_CNTRL 
		-- digital controls to/from 40-pin connector
		D_CNTRL: out std_logic_vector(5 downto 1);
			-- pin functions when interfacing with COM-35xx RF transceivers
			-- D_CNTRL1 RX_AGC2
			-- D_CNTRL2 TX_ENB
			-- D_CNTRL3 RX_TXN
			-- D_CNTRL4 RX_AGC3
			-- D_CNTRL5	RF transceiver CLKOUT

		--// ETHERNET PHY interface (MICREL KSZ9021)
		LAN_TXC: out std_logic;
		LAN_TX_CTL: out std_logic;
		LAN_TXD: out std_logic_vector(3 downto 0);
		LAN_RXC: in std_logic;
		LAN_RX_CTL: in std_logic;
		LAN_RXD: in std_logic_vector(3 downto 0);
		LAN_MCLK: out std_logic;
		LAN_MDIO: inout std_logic;
		LAN_INT_N: in std_logic;
		LAN_CLK125_NDO: in std_logic;
		LAN_RESET_N: out std_logic;

		--// USB PHY interface (SMSC USB3320 IC)
		-- Direct connection between the  component and the USB PHY.
		-- Synchronous with USB_CLKREF60 clock
		USB_CLKREF60: out std_logic;
		USB_ULPI_DATA: inout std_logic_vector(7 downto 0);
		USB_ULPI_STP: out std_logic;
		USB_ULPI_NXT: in std_logic;
		USB_ULPI_DIR: in std_logic;
		USB_RESETN: out std_logic;
		
		--// SERIAL RS-422 INTERFACE
		RS422_SERIAL_TX_CLK_OUT: out std_logic;
		RS422_SERIAL_TX_DATA_IN: in std_logic;
		RS422_SERIAL_TX_DATA_VALID_IN: in std_logic;
		RS422_SERIAL_RX_CLK_OUT: out std_logic;
		RS422_SERIAL_RX_DATA_OUT: out std_logic;
		RS422_TX_EN: out std_logic;
		RS422_RX_EN: out std_logic;
		

		--// Monitoring & Control: microcontroller interface
		-- asynchronous bus (shared with NAND flash)
		-- set address at the rising edge of UC_WEB_IN when UC_CSIB_IN = '0' and UC_ALE_IN = '1'
		-- write data the rising edge of UC_WEB_IN when UC_CSIB_IN = '0'
		-- read data when  UC_CSIB_IN = '0' and UC_REB_IN = '0'
		UC_CSIB_IN: in std_logic;  -- chip select. active low.
		UC_ALE_IN: in std_logic;  -- address latch enable. active high.
		UC_REB_IN: in std_logic;  -- read enable #
		UC_WEB_IN: in std_logic;	-- write enable #
		UC_AD: inout std_logic_vector(7 downto 0)  -- shared address/data bus

		  
			  );
end entity;

architecture Behavioral of COM1700 is
--------------------------------------------------------
--      COMPONENTS
--------------------------------------------------------
	COMPONENT HIGH_SPEED_NCO
	 Generic (
		REFCLK_FREQUENCY: integer
	 );
	PORT(
		REFCLK : IN std_logic;
		NCO_FREQ : IN std_logic_vector(19 downto 0);          
		CLK_OUTG : OUT std_logic;
		LOCKED_PLL : OUT std_logic
		);
	END COMPONENT;

    COMPONENT SPI
	 GENERIC(
		NDIV: integer range 2 to 256
	 );
    PORT(
		CLK : IN std_logic;
		ASYNC_RESET : IN std_logic;
		ADDR : IN std_logic_vector(6 downto 0);
		DATA_OUT : IN std_logic_vector(7 downto 0);
		WRITE_REQ : IN std_logic;
		READ_REQ : IN std_logic;
		SDI : IN std_logic;          
		DATA_IN : OUT std_logic_vector(7 downto 0);
		DATA_IN_CLK : OUT std_logic;
		BUSY : OUT std_logic;
		CSB : OUT std_logic;
		SCLK : OUT std_logic;
		SDO : OUT std_logic
        );
    END COMPONENT;
    

	COMPONENT P8_TO_S1_CONVERSION
	PORT(
		CLK : IN std_logic;
		SYNC_RESET : IN std_logic;
		DATA8_IN : IN std_logic_vector(7 downto 0);
		SAMPLE8_CLK_IN : IN std_logic;
		SAMPLE8_CLK_IN_REQ : OUT std_logic;
		DATA1_OUT : OUT std_logic;
		SAMPLE1_CLK_OUT : OUT std_logic;
		SAMPLE1_CLK_OUT_REQ : IN std_logic;          
		BUF_EMPTY: out std_logic
		);
	END COMPONENT;

	COMPONENT PX_TO_P8_CONVERSION
	PORT(
		CLK : IN std_logic;
		DATA_IN : IN std_logic_vector(7 downto 0);
		SAMPLE_CLK_IN : IN std_logic;
		WIDTH_IN : IN std_logic_vector(2 downto 0);          
		DATA_OUT : OUT std_logic_vector(7 downto 0);
		SAMPLE_CLK_OUT : OUT std_logic
		);
	END COMPONENT;

	COMPONENT CROSS_CLK_DOMAINS_NODATALOSS
	PORT(
		CLK_A : IN std_logic;
		CLK_B : IN std_logic;
		ASYNC_RESET : IN std_logic;
		DATA_IN : IN std_logic_vector(7 downto 0);
		SAMPLE_CLK_IN : IN std_logic;
		SOF_IN : IN std_logic;
		EOF_IN : IN std_logic;
		SAMPLE_CLK_OUT_REQ : IN std_logic;          
		SAMPLE_CLK_IN_REQ : OUT std_logic;
		DATA_OUT : OUT std_logic_vector(7 downto 0);
		SAMPLE_CLK_OUT : OUT std_logic;
		SOF_OUT : OUT std_logic;
		EOF_OUT : OUT std_logic
		);
	END COMPONENT;

	COMPONENT AD9747_DAC_DRIVER2
	PORT(
		CLK60G : IN std_logic;
		CLK : IN std_logic;
		DAC1_DIN : IN std_logic_vector(15 downto 0);
		DAC2_DIN : IN std_logic_vector(15 downto 0);          
		DAC1_DOUT : OUT std_logic_vector(15 downto 0);
		DAC2_DOUT : OUT std_logic_vector(15 downto 0);
		DAC_CLKOUT_P : OUT std_logic;
		DAC_CLKOUT_N : OUT std_logic
		);
	END COMPONENT;

	COMPONENT CROSS_CLK_DOMAINS
	generic (
		NBITS: in integer
	);
	PORT(
		CLK_A : in std_logic;				-- 'from' clock domain
		CLK_B : in std_logic;				-- 'to' clock domain
		ASYNC_RESET: in std_logic;
		DATA_IN: in std_logic_vector((NBITS-1) downto 0);
		DATA_OUT: out std_logic_vector((NBITS-1) downto 0)
		);
	END COMPONENT;

	COMPONENT CLEAN_CLK2
	PORT(
		CLK : IN std_logic;
		SYNC_RESET : IN std_logic;
		CLK_PULSE_IN : IN std_logic;          
		CLK_50DUTY_OUT : OUT std_logic;
		CLK_PULSE_OUT : OUT std_logic;
		ERROR: out std_logic
		);
	END COMPONENT;

	COMPONENT NCO32
	PORT(
		CLK : IN std_logic;
		SYNC_RESET : IN std_logic;
		NOMINAL_FREQUENCY : IN std_logic_vector(31 downto 0);
		PHASE_OFFSET : IN std_logic_vector(31 downto 0);
		PHASE_OFFSET_SAMPLE_CLK : IN std_logic;          
		PHASE : OUT std_logic_vector(31 downto 0)
		);
	END COMPONENT;

	COMPONENT ELASTIC_BUFFER32b
	PORT(
		CLK : IN std_logic;
		DATA_IN : IN std_logic;
		SAMPLE_CLK_IN : IN std_logic;
		SAMPLE_CLK_OUT_REQ : IN std_logic;          
		SAMPLE_CLK_IN_REQ : OUT std_logic;
		DATA_OUT : OUT std_logic;
		SAMPLE_CLK_OUT : OUT std_logic
		);
	END COMPONENT;

	COMPONENT ELASTIC_BUFFER128b
	PORT(
		CLK : IN std_logic;
		DATA_IN : IN std_logic;
		SAMPLE_CLK_IN : IN std_logic;
		SAMPLE_CLK_OUT_REQ : IN std_logic;          
		SAMPLE_CLK_IN_REQ : OUT std_logic;
		DATA_OUT : OUT std_logic;
		SAMPLE_CLK_OUT : OUT std_logic
		);
	END COMPONENT;

	COMPONENT DNA_ID
	PORT(
		CLK : IN std_logic;
		ASYNC_RESET : IN std_logic;          
		ID_OUT : OUT std_logic_vector(56 downto 0);
		ID_VALID_OUT : OUT std_logic;
		ID_SAMPLE_CLK_OUT : OUT std_logic
		);
	END COMPONENT;

	COMPONENT USB20ULPI_CLK2PHY
	PORT(
		ASYNC_RESET : IN std_logic;
		CLK_P : IN std_logic;
		USB_CLK60G : IN std_logic;
		USB_ULPI_NXT : IN std_logic;
		USB_ULPI_DIR : IN std_logic;
		DATA1_OUT_SAMPLE_CLK_REQ : IN std_logic;
		DATA1_IN : IN std_logic_vector(7 downto 0);
		DATA1_IN_SAMPLE_CLK : IN std_logic;
		DATA2_OUT_SAMPLE_CLK_REQ : IN std_logic;
		DATA2_IN : IN std_logic_vector(7 downto 0);
		DATA2_IN_SAMPLE_CLK : IN std_logic;    
		USB_ULPI_DATA : INOUT std_logic_vector(7 downto 0);      
		USB_ULPI_STP : OUT std_logic;
		DATA1_OUT : OUT std_logic_vector(7 downto 0);
		DATA1_OUT_SAMPLE_CLK : OUT std_logic;
		DATA1_OUT_BUFFER_EMPTY : OUT std_logic;
		DATA1_IN_SAMPLE_CLK_REQ : OUT std_logic;
		DATA2_OUT : OUT std_logic_vector(7 downto 0);
		DATA2_OUT_SAMPLE_CLK : OUT std_logic;
		DATA2_OUT_BUFFER_EMPTY : OUT std_logic;
		DATA2_IN_SAMPLE_CLK_REQ : OUT std_logic;
		PHY_VENDOR_ID : OUT std_logic_vector(7 downto 0);
		CS1 : OUT std_logic_vector(7 downto 0);
		CS1_CLK : OUT std_logic;
		CS2 : OUT std_logic_vector(7 downto 0);
		CS2_CLK : OUT std_logic;
		CS3 : OUT std_logic_vector(7 downto 0);
		CS3_CLK : OUT std_logic;
		T1 : OUT std_logic;
		T1_CLK : OUT std_logic;
		T2 : OUT std_logic;
		T2_CLK : OUT std_logic;
		USB_TP : OUT std_logic_vector(20 downto 1)
		);
	END COMPONENT;

	COMPONENT COM5401
	GENERIC (
		MII_SEL: std_logic;  
		PHY_ADDR: std_logic_vector(4 downto 0);
		CLK_FREQUENCY: integer; 
		SIMULATION: std_logic
	);
	PORT(
		CLK : IN std_logic;
		SYNC_RESET : IN std_logic;
		MAC_TX_CONFIG : IN std_logic_vector(15 downto 0);
		MAC_RX_CONFIG : IN std_logic_vector(15 downto 0);
		MAC_ADDR : IN std_logic_vector(47 downto 0);
		PHY_CONFIG_CHANGE : IN std_logic;
		PHY_RESET : IN std_logic;
		SPEED : IN std_logic_vector(1 downto 0);
		DUPLEX : IN std_logic;
		TEST_MODE : IN std_logic_vector(1 downto 0);
		POWER_DOWN : IN std_logic;
		CLK_SKEW: in std_logic_vector(9 downto 0);
		MAC_TX_DATA : IN std_logic_vector(7 downto 0);
		MAC_TX_DATA_VALID : IN std_logic;
		MAC_TX_EOF : IN std_logic;
		MAC_RX_CTS : IN std_logic;
		RXC : IN std_logic;
		RXD : IN std_logic_vector(3 downto 0);
		RX_CTL : IN std_logic;
		MII_TX_CLK : IN std_logic;
		GMII_MII_CRS : IN std_logic;
		GMII_MII_COL : IN std_logic;
		GMII_MII_RX_CLK : IN std_logic;
		GMII_MII_RXD : IN std_logic_vector(7 downto 0);
		GMII_MII_RX_DV : IN std_logic;
		GMII_MII_RX_ER : IN std_logic;    
		MDIO : INOUT std_logic;      
		MAC_TX_CTS : OUT std_logic;
		MAC_RX_DATA : OUT std_logic_vector(7 downto 0);
		MAC_RX_DATA_VALID : OUT std_logic;
		MAC_RX_SOF : OUT std_logic;
		MAC_RX_EOF : OUT std_logic;
		TXC : OUT std_logic;
		TXD : OUT std_logic_vector(3 downto 0);
		TX_CTL : OUT std_logic;
		RESET_N : OUT std_logic;
		MCLK : OUT std_logic;
		GMII_TX_CLK : OUT std_logic;
		GMII_MII_TXD : OUT std_logic_vector(7 downto 0);
		GMII_MII_TX_EN : OUT std_logic;
		GMII_MII_TX_ER : OUT std_logic;
		LINK_STATUS : OUT std_logic;
		SPEED_STATUS : OUT std_logic_vector(1 downto 0);
		DUPLEX_STATUS : OUT std_logic;
		PHY_ID: out std_logic_vector(15 downto 0);
		N_RX_FRAMES: out  std_logic_vector(15 downto 0);
		N_RX_BAD_CRCS: out  std_logic_vector(15 downto 0);
		N_RX_FRAMES_TOO_SHORT: out  std_logic_vector(15 downto 0);
		N_RX_FRAMES_TOO_LONG: out  std_logic_vector(15 downto 0);
		N_RX_WRONG_ADDR: out  std_logic_vector(15 downto 0);
		N_RX_LENGTH_ERRORS: out  std_logic_vector(15 downto 0);
		CS_CLKG: out std_logic;	-- synchronous clock for the comscope traces
		CS1: out std_logic_vector(7 downto 0);
		CS1_CLK: out std_logic;
		CS2: out std_logic_vector(7 downto 0);
		CS2_CLK: out std_logic;
		TP: out std_logic_vector(10 downto 1)
		);
	END COMPONENT;

	COMPONENT COM5402
	GENERIC (
		NUDPTX: integer;
		NUDPRX: integer;
		NTCPSTREAMS: integer;  
		CLK_FREQUENCY: integer;
		TX_IDLE_TIMEOUT: integer;	
		SIMULATION: std_logic
	);	
	PORT(
		CLK : IN std_logic;
		ASYNC_RESET : IN std_logic;
		SYNC_RESET : IN std_logic;
		MAC_ADDR : IN std_logic_vector(47 downto 0);
		IPv4_ADDR : IN std_logic_vector(31 downto 0);
		IPv6_ADDR : IN std_logic_vector(127 downto 0);
		MULTICAST_IP_ADDR: in std_logic_vector(31 downto 0);
		SUBNET_MASK: in std_logic_vector(31 downto 0);
		GATEWAY_IP_ADDR: in std_logic_vector(31 downto 0);
		CONNECTION_RESET: in std_logic_vector((NTCPSTREAMS-1) downto 0);
		MAC_TX_CTS : IN std_logic;
		MAC_RX_DATA : IN std_logic_vector(7 downto 0);
		MAC_RX_DATA_VALID : IN std_logic;
		MAC_RX_SOF : IN std_logic;
		MAC_RX_EOF : IN std_logic;          
		MAC_TX_DATA : OUT std_logic_vector(7 downto 0);
		MAC_TX_DATA_VALID : OUT std_logic;
		MAC_TX_SOF : OUT std_logic;
		MAC_TX_EOF : OUT std_logic;
		UDP_RX_DATA: out std_logic_vector(7 downto 0);
		UDP_RX_DATA_VALID: out std_logic;
		UDP_RX_SOF: out std_logic;	
		UDP_RX_EOF: out std_logic;	
		UDP_RX_DEST_PORT_NO: in std_logic_vector(15 downto 0);
		UDP_TX_DATA: in std_logic_vector(7 downto 0);
		UDP_TX_DATA_VALID: in std_logic;
		UDP_TX_SOF: in std_logic;	
		UDP_TX_EOF: in std_logic;	
		UDP_TX_CTS: out std_logic;	
		UDP_TX_ACK: out std_logic;	
		UDP_TX_NAK: out std_logic;	
		UDP_TX_DEST_IP_ADDR: in std_logic_vector(127 downto 0);
		UDP_TX_DEST_PORT_NO: in std_logic_vector(15 downto 0);
		UDP_TX_SOURCE_PORT_NO: in std_logic_vector(15 downto 0);
		TCP_RX_DATA: out SLV8xNTCPSTREAMStype;
		TCP_RX_DATA_VALID: out std_logic_vector((NTCPSTREAMS-1) downto 0);
		TCP_RX_RTS: out std_logic_vector((NTCPSTREAMS-1) downto 0);	-- Ready To Send
		TCP_RX_CTS: in std_logic_vector((NTCPSTREAMS-1) downto 0);	-- Clear To Send
		TCP_TX_DATA: in SLV8xNTCPSTREAMStype;
		TCP_TX_DATA_VALID: in std_logic_vector((NTCPSTREAMS-1) downto 0);
		TCP_TX_CTS: out std_logic_vector((NTCPSTREAMS-1) downto 0);	
		TCP_CONNECTED_FLAG: out std_logic_vector((NTCPSTREAMS-1) downto 0);
		CS1 : OUT std_logic_vector(7 downto 0);
		CS1_CLK : OUT std_logic;
		CS2 : OUT std_logic_vector(7 downto 0);
		CS2_CLK : OUT std_logic;
		TP : OUT std_logic_vector(10 downto 1)
		);
	END COMPONENT;

	COMPONENT ELASTIC_BUFFER_NRAMB
	GENERIC (
		NRAMB: integer;
		NBITS: integer;
		READ_AHEAD_1ADDRESS: std_logic
	);	
	PORT(
		CLK : IN std_logic;
		SYNC_RESET : IN std_logic;
		DATA_IN : IN std_logic_vector(8 downto 0);
		SAMPLE_CLK_IN : IN std_logic;
		SAMPLE_CLK_OUT_REQ : IN std_logic;          
		SAMPLE_CLK_IN_REQ : OUT std_logic;
		DATA_OUT : OUT std_logic_vector(8 downto 0);
		SAMPLE_CLK_OUT : OUT std_logic;
		SAMPLE_CLK_OUT_E : OUT std_logic;
		BUFFER_EMPTY: out std_logic
		);
	END COMPONENT;

	COMPONENT COMSCOPE
	generic (
		-- select features as needed: number of traces, widths
		-- One must re-synthesize after changing these choices.
		-- enable individual traces. '1' for enable, '0' for disable
		TRACE1_ENABLED: in std_logic;	
		TRACE2_ENABLED: in std_logic;	
		TRACE3_ENABLED: in std_logic;	
		TRACE4_ENABLED: in std_logic
	);	
	PORT(
		CLK : IN std_logic;
		ASYNC_RESET : IN std_logic;
		SIGNED_REPRESENTATION_U_SIGNED_N : IN std_logic;
		REG237 : IN std_logic_vector(7 downto 0);
		REG238 : IN std_logic_vector(7 downto 0);
		REG239 : IN std_logic_vector(7 downto 0);
		REG240 : IN std_logic_vector(7 downto 0);
		REG241 : IN std_logic_vector(7 downto 0);
		REG242 : IN std_logic_vector(7 downto 0);
		REG243 : IN std_logic_vector(7 downto 0);
		REG244 : IN std_logic_vector(7 downto 0);
		REG245 : IN std_logic_vector(7 downto 0);
		REG246 : IN std_logic_vector(7 downto 0);
		REG247 : IN std_logic_vector(7 downto 0);
		REG248 : IN std_logic_vector(7 downto 0);
		REG249 : IN std_logic_vector(7 downto 0);
		SIGNAL_1_BIT_1_1 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_1_1 : IN std_logic;
		SIGNAL_1_BIT_1_2 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_1_2 : IN std_logic;
		SIGNAL_1_BIT_1_3 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_1_3 : IN std_logic;
		SIGNAL_1_BIT_1_4 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_1_4 : IN std_logic;
--		SIGNAL_2_BIT_1_1 : IN std_logic_vector(1 downto 0);
--		SIGNAL_2_BIT_SAMPLE_CLK_1_1 : IN std_logic;
--		SIGNAL_2_BIT_1_2 : IN std_logic_vector(1 downto 0);
--		SIGNAL_2_BIT_SAMPLE_CLK_1_2 : IN std_logic;
--		SIGNAL_2_BIT_1_3 : IN std_logic_vector(1 downto 0);
--		SIGNAL_2_BIT_SAMPLE_CLK_1_3 : IN std_logic;
--		SIGNAL_2_BIT_1_4 : IN std_logic_vector(1 downto 0);
--		SIGNAL_2_BIT_SAMPLE_CLK_1_4 : IN std_logic;
--		SIGNAL_4_BIT_1_1 : IN std_logic_vector(3 downto 0);
--		SIGNAL_4_BIT_SAMPLE_CLK_1_1 : IN std_logic;
--		SIGNAL_4_BIT_1_2 : IN std_logic_vector(3 downto 0);
--		SIGNAL_4_BIT_SAMPLE_CLK_1_2 : IN std_logic;
--		SIGNAL_4_BIT_1_3 : IN std_logic_vector(3 downto 0);
--		SIGNAL_4_BIT_SAMPLE_CLK_1_3 : IN std_logic;
--		SIGNAL_4_BIT_1_4 : IN std_logic_vector(3 downto 0);
--		SIGNAL_4_BIT_SAMPLE_CLK_1_4 : IN std_logic;
		SIGNAL_8_BIT_1_1 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_1_1 : IN std_logic;
		SIGNAL_8_BIT_1_2 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_1_2 : IN std_logic;
		SIGNAL_8_BIT_1_3 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_1_3 : IN std_logic;
		SIGNAL_8_BIT_1_4 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_1_4 : IN std_logic;
--		SIGNAL_16_BIT_1_1 : IN std_logic_vector(15 downto 0);
--		SIGNAL_16_BIT_SAMPLE_CLK_1_1 : IN std_logic;
--		SIGNAL_16_BIT_1_2 : IN std_logic_vector(15 downto 0);
--		SIGNAL_16_BIT_SAMPLE_CLK_1_2 : IN std_logic;
--		SIGNAL_16_BIT_1_3 : IN std_logic_vector(15 downto 0);
--		SIGNAL_16_BIT_SAMPLE_CLK_1_3 : IN std_logic;
--		SIGNAL_16_BIT_1_4 : IN std_logic_vector(15 downto 0);
--		SIGNAL_16_BIT_SAMPLE_CLK_1_4 : IN std_logic;
		SIGNAL_1_BIT_2_1 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_2_1 : IN std_logic;
		SIGNAL_1_BIT_2_2 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_2_2 : IN std_logic;
		SIGNAL_1_BIT_2_3 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_2_3 : IN std_logic;
		SIGNAL_1_BIT_2_4 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_2_4 : IN std_logic;
--		SIGNAL_2_BIT_2_1 : IN std_logic_vector(1 downto 0);
--		SIGNAL_2_BIT_SAMPLE_CLK_2_1 : IN std_logic;
--		SIGNAL_2_BIT_2_2 : IN std_logic_vector(1 downto 0);
--		SIGNAL_2_BIT_SAMPLE_CLK_2_2 : IN std_logic;
--		SIGNAL_2_BIT_2_3 : IN std_logic_vector(1 downto 0);
--		SIGNAL_2_BIT_SAMPLE_CLK_2_3 : IN std_logic;
--		SIGNAL_2_BIT_2_4 : IN std_logic_vector(1 downto 0);
--		SIGNAL_2_BIT_SAMPLE_CLK_2_4 : IN std_logic;
--		SIGNAL_4_BIT_2_1 : IN std_logic_vector(3 downto 0);
--		SIGNAL_4_BIT_SAMPLE_CLK_2_1 : IN std_logic;
--		SIGNAL_4_BIT_2_2 : IN std_logic_vector(3 downto 0);
--		SIGNAL_4_BIT_SAMPLE_CLK_2_2 : IN std_logic;
--		SIGNAL_4_BIT_2_3 : IN std_logic_vector(3 downto 0);
--		SIGNAL_4_BIT_SAMPLE_CLK_2_3 : IN std_logic;
--		SIGNAL_4_BIT_2_4 : IN std_logic_vector(3 downto 0);
--		SIGNAL_4_BIT_SAMPLE_CLK_2_4 : IN std_logic;
		SIGNAL_8_BIT_2_1 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_2_1 : IN std_logic;
		SIGNAL_8_BIT_2_2 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_2_2 : IN std_logic;
		SIGNAL_8_BIT_2_3 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_2_3 : IN std_logic;
		SIGNAL_8_BIT_2_4 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_2_4 : IN std_logic;
--		SIGNAL_16_BIT_2_1 : IN std_logic_vector(15 downto 0);
--		SIGNAL_16_BIT_SAMPLE_CLK_2_1 : IN std_logic;
--		SIGNAL_16_BIT_2_2 : IN std_logic_vector(15 downto 0);
--		SIGNAL_16_BIT_SAMPLE_CLK_2_2 : IN std_logic;
--		SIGNAL_16_BIT_2_3 : IN std_logic_vector(15 downto 0);
--		SIGNAL_16_BIT_SAMPLE_CLK_2_3 : IN std_logic;
--		SIGNAL_16_BIT_2_4 : IN std_logic_vector(15 downto 0);
--		SIGNAL_16_BIT_SAMPLE_CLK_2_4 : IN std_logic;
		SIGNAL_1_BIT_3_1 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_3_1 : IN std_logic;
		SIGNAL_1_BIT_3_2 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_3_2 : IN std_logic;
		SIGNAL_1_BIT_3_3 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_3_3 : IN std_logic;
		SIGNAL_1_BIT_3_4 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_3_4 : IN std_logic;
--		SIGNAL_2_BIT_3_1 : IN std_logic_vector(1 downto 0);
--		SIGNAL_2_BIT_SAMPLE_CLK_3_1 : IN std_logic;
--		SIGNAL_2_BIT_3_2 : IN std_logic_vector(1 downto 0);
--		SIGNAL_2_BIT_SAMPLE_CLK_3_2 : IN std_logic;
--		SIGNAL_2_BIT_3_3 : IN std_logic_vector(1 downto 0);
--		SIGNAL_2_BIT_SAMPLE_CLK_3_3 : IN std_logic;
--		SIGNAL_2_BIT_3_4 : IN std_logic_vector(1 downto 0);
--		SIGNAL_2_BIT_SAMPLE_CLK_3_4 : IN std_logic;
--		SIGNAL_4_BIT_3_1 : IN std_logic_vector(3 downto 0);
--		SIGNAL_4_BIT_SAMPLE_CLK_3_1 : IN std_logic;
--		SIGNAL_4_BIT_3_2 : IN std_logic_vector(3 downto 0);
--		SIGNAL_4_BIT_SAMPLE_CLK_3_2 : IN std_logic;
--		SIGNAL_4_BIT_3_3 : IN std_logic_vector(3 downto 0);
--		SIGNAL_4_BIT_SAMPLE_CLK_3_3 : IN std_logic;
--		SIGNAL_4_BIT_3_4 : IN std_logic_vector(3 downto 0);
--		SIGNAL_4_BIT_SAMPLE_CLK_3_4 : IN std_logic;
		SIGNAL_8_BIT_3_1 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_3_1 : IN std_logic;
		SIGNAL_8_BIT_3_2 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_3_2 : IN std_logic;
		SIGNAL_8_BIT_3_3 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_3_3 : IN std_logic;
		SIGNAL_8_BIT_3_4 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_3_4 : IN std_logic;
--		SIGNAL_16_BIT_3_1 : IN std_logic_vector(15 downto 0);
--		SIGNAL_16_BIT_SAMPLE_CLK_3_1 : IN std_logic;
--		SIGNAL_16_BIT_3_2 : IN std_logic_vector(15 downto 0);
--		SIGNAL_16_BIT_SAMPLE_CLK_3_2 : IN std_logic;
--		SIGNAL_16_BIT_3_3 : IN std_logic_vector(15 downto 0);
--		SIGNAL_16_BIT_SAMPLE_CLK_3_3 : IN std_logic;
--		SIGNAL_16_BIT_3_4 : IN std_logic_vector(15 downto 0);
--		SIGNAL_16_BIT_SAMPLE_CLK_3_4 : IN std_logic;
		SIGNAL_1_BIT_4_1 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_4_1 : IN std_logic;
		SIGNAL_1_BIT_4_2 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_4_2 : IN std_logic;
		SIGNAL_1_BIT_4_3 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_4_3 : IN std_logic;
		SIGNAL_1_BIT_4_4 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_4_4 : IN std_logic;
--		SIGNAL_2_BIT_4_1 : IN std_logic_vector(1 downto 0);
--		SIGNAL_2_BIT_SAMPLE_CLK_4_1 : IN std_logic;
--		SIGNAL_2_BIT_4_2 : IN std_logic_vector(1 downto 0);
--		SIGNAL_2_BIT_SAMPLE_CLK_4_2 : IN std_logic;
--		SIGNAL_2_BIT_4_3 : IN std_logic_vector(1 downto 0);
--		SIGNAL_2_BIT_SAMPLE_CLK_4_3 : IN std_logic;
--		SIGNAL_2_BIT_4_4 : IN std_logic_vector(1 downto 0);
--		SIGNAL_2_BIT_SAMPLE_CLK_4_4 : IN std_logic;
--		SIGNAL_4_BIT_4_1 : IN std_logic_vector(3 downto 0);
--		SIGNAL_4_BIT_SAMPLE_CLK_4_1 : IN std_logic;
--		SIGNAL_4_BIT_4_2 : IN std_logic_vector(3 downto 0);
--		SIGNAL_4_BIT_SAMPLE_CLK_4_2 : IN std_logic;
--		SIGNAL_4_BIT_4_3 : IN std_logic_vector(3 downto 0);
--		SIGNAL_4_BIT_SAMPLE_CLK_4_3 : IN std_logic;
--		SIGNAL_4_BIT_4_4 : IN std_logic_vector(3 downto 0);
--		SIGNAL_4_BIT_SAMPLE_CLK_4_4 : IN std_logic;
		SIGNAL_8_BIT_4_1 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_4_1 : IN std_logic;
		SIGNAL_8_BIT_4_2 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_4_2 : IN std_logic;
		SIGNAL_8_BIT_4_3 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_4_3 : IN std_logic;
		SIGNAL_8_BIT_4_4 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_4_4 : IN std_logic;
--		SIGNAL_16_BIT_4_1 : IN std_logic_vector(15 downto 0);
--		SIGNAL_16_BIT_SAMPLE_CLK_4_1 : IN std_logic;
--		SIGNAL_16_BIT_4_2 : IN std_logic_vector(15 downto 0);
--		SIGNAL_16_BIT_SAMPLE_CLK_4_2 : IN std_logic;
--		SIGNAL_16_BIT_4_3 : IN std_logic_vector(15 downto 0);
--		SIGNAL_16_BIT_SAMPLE_CLK_4_3 : IN std_logic;
--		SIGNAL_16_BIT_4_4 : IN std_logic_vector(15 downto 0);
--		SIGNAL_16_BIT_SAMPLE_CLK_4_4 : IN std_logic;
		TRIGGER_1_BIT_1 : IN std_logic;
		TRIGGER_1_BIT_SAMPLE_CLK_1 : IN std_logic;
		TRIGGER_1_BIT_2 : IN std_logic;
		TRIGGER_1_BIT_SAMPLE_CLK_2 : IN std_logic;
		TRIGGER_1_BIT_3 : IN std_logic;
		TRIGGER_1_BIT_SAMPLE_CLK_3 : IN std_logic;
		TRIGGER_1_BIT_4 : IN std_logic;
		TRIGGER_1_BIT_SAMPLE_CLK_4 : IN std_logic;
--		TRIGGER_2_BIT_1 : IN std_logic_vector(1 downto 0);
--		TRIGGER_2_BIT_SAMPLE_CLK_1 : IN std_logic;
--		TRIGGER_2_BIT_2 : IN std_logic_vector(1 downto 0);
--		TRIGGER_2_BIT_SAMPLE_CLK_2 : IN std_logic;
--		TRIGGER_2_BIT_3 : IN std_logic_vector(1 downto 0);
--		TRIGGER_2_BIT_SAMPLE_CLK_3 : IN std_logic;
--		TRIGGER_2_BIT_4 : IN std_logic_vector(1 downto 0);
--		TRIGGER_2_BIT_SAMPLE_CLK_4 : IN std_logic;
--		TRIGGER_4_BIT_1 : IN std_logic_vector(3 downto 0);
--		TRIGGER_4_BIT_SAMPLE_CLK_1 : IN std_logic;
--		TRIGGER_4_BIT_2 : IN std_logic_vector(3 downto 0);
--		TRIGGER_4_BIT_SAMPLE_CLK_2 : IN std_logic;
--		TRIGGER_4_BIT_3 : IN std_logic_vector(3 downto 0);
--		TRIGGER_4_BIT_SAMPLE_CLK_3 : IN std_logic;
--		TRIGGER_4_BIT_4 : IN std_logic_vector(3 downto 0);
--		TRIGGER_4_BIT_SAMPLE_CLK_4 : IN std_logic;
		TRIGGER_8_BIT_1 : IN std_logic_vector(7 downto 0);
		TRIGGER_8_BIT_SAMPLE_CLK_1 : IN std_logic;
		TRIGGER_8_BIT_2 : IN std_logic_vector(7 downto 0);
		TRIGGER_8_BIT_SAMPLE_CLK_2 : IN std_logic;
		TRIGGER_8_BIT_3 : IN std_logic_vector(7 downto 0);
		TRIGGER_8_BIT_SAMPLE_CLK_3 : IN std_logic;
		TRIGGER_8_BIT_4 : IN std_logic_vector(7 downto 0);
		TRIGGER_8_BIT_SAMPLE_CLK_4 : IN std_logic;
--		TRIGGER_16_BIT_1 : IN std_logic_vector(15 downto 0);
--		TRIGGER_16_BIT_SAMPLE_CLK_1 : IN std_logic;
--		TRIGGER_16_BIT_2 : IN std_logic_vector(15 downto 0);
--		TRIGGER_16_BIT_SAMPLE_CLK_2 : IN std_logic;
--		TRIGGER_16_BIT_3 : IN std_logic_vector(15 downto 0);
--		TRIGGER_16_BIT_SAMPLE_CLK_3 : IN std_logic;
--		TRIGGER_16_BIT_4 : IN std_logic_vector(15 downto 0);
--		TRIGGER_16_BIT_SAMPLE_CLK_4 : IN std_logic;
		TRIGGER_REARM_TOGGLE : IN std_logic;
		FORCE_TRIGGER_TOGGLE : IN std_logic;
		SREG250_READ_TOGGLE : IN std_logic;
		START_CAPTURE_TOGGLE : IN std_logic;          
		REG250 : OUT std_logic_vector(7 downto 0);
		REG251 : OUT std_logic_vector(7 downto 0);
		TP : OUT std_logic_vector(10 downto 1)
		);
	END COMPONENT;

	component SIM2OUTFILE is
		 Generic (
			FILENAME: string
		 );
		 Port ( 
			--// CLK, RESET
			CLK: in std_logic;
				-- Reference clock
			DATA1: in std_logic_vector(11 downto 0);
			DATA2: in std_logic_vector(11 downto 0);
			DATA3: in std_logic_vector(11 downto 0);
			SAMPLE_CLK_IN: in std_logic
				-- DATA_IN is read at the rising edge of CLK when SAMPLE_CLK_IN = '1'

	);
	END COMPONENT;
	
--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------
-- Suffix _D indicates a one CLK delayed version of the net with the same name
-- Suffix _X indicates an extended precision version of the net with the same name
-- Suffix _N indicates an inverted version of the net with the same name
-- Suffix G indicates a global clock

--// CLOCKS, RESETS
signal CLKREFg: std_logic := '0';
signal PLL1_LOCKED: std_logic := '0';
signal ADC_CLK_CLKFB1: std_logic := '0';
signal CLK_ADCg: std_logic := '0';
signal CLK_ADCg_N: std_logic := '0';
signal ADC_CLK: std_logic := '0';
signal CLK_P: std_logic := '0';
signal CLK60: std_logic := '0';
signal CLK60G: std_logic := '0';
signal PLL2_LOCKED: std_logic := '0';
signal CLK60_CLKFB1: std_logic := '0';
signal DAC_CLK_FREQ: std_logic_vector(19 downto 0) := (others => '0');
signal CLK_DAC: std_logic := '0';
signal CLK_DACg: std_logic := '0';
signal PLL_LOCKED_TX: std_logic := '0';
signal SYNC_RESET_CLK_P: std_logic := '0';
signal SYNC_RESET_CLK_DAC: std_logic := '0';
signal RESET_COUNTER: std_logic_vector(21 downto 0) := (others => '0');	-- initialize for simulations
	-- spans 2^21/26MHz = 80ms
signal ASYNC_RESETN: std_logic := '0';
signal ASYNC_RESET: std_logic := '0';
signal ASYNC_RESET_D: std_logic := '0';
signal ASYNC_RESET2_D: std_logic := '0';
	-- asynchronous reset

--// DACs ----------------------------------------------------
signal DAC1: std_logic_vector(11 downto 0) := (others => '0');	-- user-defined output samples. 2's complement
signal DAC2: std_logic_vector(11 downto 0) := (others => '0');

--//-- RECLOCKED INPUTS ---------------------
signal ADC_DATA_I_IN: std_logic_vector(13 downto 0) := (others => '0');
signal ADC_DATA_Q_IN: std_logic_vector(13 downto 0) := (others => '0');

--// USB INTERFACE ---------------------------------------
signal USB_RX_DATA2: std_logic_vector(7 downto 0) := (others => '0'); 
signal USB_RX_SAMPLE2_CLK: std_logic:= '0';
signal USB_RX_SAMPLE2_CLK_REQ: std_logic:= '0';
signal USB_TX_DATA2: std_logic_vector(7 downto 0) := (others => '0'); 
signal USB_TX_DATA2_SAMPLE_CLK: std_logic:= '0';
signal USB_TX_DATA2_SAMPLE_CLK_REQ: std_logic:= '0';

--// USB Interface for M&C
signal USB_ULPI_STP_LOCAL: std_logic:= '0';
signal RX_USB: std_logic_vector(7 downto 0) := (others => '0'); 
signal RX_USB_SAMPLE_CLK_REQ: std_logic:= '0';
signal RX_USB_BUFFER_EMPTY: std_logic:= '0';
signal RX_USB_BUFFER_EMPTY_D: std_logic:= '0';
signal RX_USB_BUFFER_EMPTY_D2: std_logic:= '0';
signal SREG254_READ_TOGGLE_D: std_logic:= '0';
signal SREG254_READ_TOGGLE_D2: std_logic:= '0';
signal TX_USB: std_logic_vector(7 downto 0) := (others => '0'); 
signal TX_USB_SAMPLE_CLK: std_logic:= '0';
signal TX_USB_TOGGLE: std_logic:= '0';
signal TX_USB_TOGGLE_D: std_logic:= '0';
signal TX_USB_TOGGLE_D2: std_logic:= '0';
signal USB_PHY_VENDOR_ID: std_logic_vector(7 downto 0) := (others => '0');

--//-- 10/100/1000 Mbps Ethernet -------
signal CLK_SKEW: std_logic_vector(9 downto 0) := (others => '0');
--signal LAN1_CLK125G: std_logic:= '0';
signal LAN1_MAC_ADDR: std_logic_vector(47 downto 0) := (others => '0');
signal COM5401_TP: std_logic_vector(10 downto 1) := (others => '0');
signal LAN1_PHY_ID: std_logic_vector(15 downto 0) := (others => '0');
signal LAN1_MAC_RX_DATA: std_logic_vector(7 downto 0) := (others => '0');
signal LAN1_MAC_RX_DATA_VALID: std_logic:= '0';
signal LAN1_MAC_RX_SOF: std_logic:= '0';
signal LAN1_MAC_RX_EOF: std_logic:= '0';
signal LAN1_MAC_TX_DATA: std_logic_vector(7 downto 0) := (others => '0');
signal LAN1_MAC_TX_DATA_VALID: std_logic:= '0';
signal LAN1_MAC_TX_SOF: std_logic:= '0';
signal LAN1_MAC_TX_EOF: std_logic:= '0';
signal LAN1_MAC_TX_CTS: std_logic:= '0';
signal LAN1_LINK_STATUS: std_logic:= '0';
signal LAN1_SPEED_STATUS: std_logic_vector(1 downto 0) := (others => '0');
signal LAN1_DUPLEX_STATUS: std_logic:= '0';

constant NTCPSTREAMS: integer range 0 to 255 := 2;  
signal COM5402_TP: std_logic_vector(10 downto 1) := (others => '0');
signal IP1_ADDRv4: std_logic_vector(31 downto 0) := (others => '0');	
signal IP1_SUBNET_MASK: std_logic_vector(31 downto 0) := (others => '0');	
signal IP1_GATEWAY: std_logic_vector(31 downto 0) := (others => '0');	
signal LAN1_TCP_RX_DATA: SLV8xNTCPSTREAMStype;
signal LAN1_TCP_RX_DATA_VALID: std_logic_vector((NTCPSTREAMS-1) downto 0) := (others => '0');
signal LAN1_TCP_RX_RTS: std_logic_vector((NTCPSTREAMS-1) downto 0) := (others => '0');
signal LAN1_TCP_RX_CTS: std_logic_vector((NTCPSTREAMS-1) downto 0) := (others => '0');
signal LAN1_TCP_TX_DATA: SLV8xNTCPSTREAMStype;
signal LAN1_TCP_TX_DATA_VALID: std_logic_vector((NTCPSTREAMS-1) downto 0) := (others => '0');
signal LAN1_TCP_TX_CTS: std_logic_vector((NTCPSTREAMS-1) downto 0) := (others => '0');
signal TCP_CONNECTED_FLAG: std_logic_vector((NTCPSTREAMS-1) downto 0) := (others => '0');

--// LAN TCP-IP Interface for M&C (port 1028)
signal RX_LAN: std_logic_vector(7 downto 0) := (others => '0'); 
signal RX_LAN_SAMPLE_CLK_REQ: std_logic;
signal RX_LAN_BUFFER_EMPTY: std_logic;
signal RX_LAN_BUFFER_EMPTY_D: std_logic;
signal RX_LAN_BUFFER_EMPTY_D2: std_logic;
signal TX_LAN_TOGGLE: std_logic;
signal TX_LAN_TOGGLE_D: std_logic;
signal TX_LAN_TOGGLE_D2: std_logic;
signal LAN_RX_DATA1: std_logic_vector(7 downto 0) := (others => '0');
signal LAN_RX_DATA1_X: std_logic_vector(8 downto 0) := (others => '0');

--// ComScope traces
signal S1_1: std_logic_vector(7 downto 0) := (others => '0');	
signal S1_1_CLK: std_logic := '0';
signal S1_1B_CLK: std_logic := '0';
signal S1_2: std_logic_vector(7 downto 0) := (others => '0');	
signal S1_2_CLK: std_logic := '0';
signal S1_2B_CLK: std_logic := '0';
signal S1_3: std_logic_vector(7 downto 0) := (others => '0');	
signal S1_3_CLK: std_logic := '0';
signal S1_3B_CLK: std_logic := '0';
signal S1_4: std_logic_vector(7 downto 0) := (others => '0');	
signal S1_4_CLK: std_logic := '0';
signal S1_4B_CLK: std_logic := '0';
signal S2_1: std_logic_vector(7 downto 0) := (others => '0');	
signal S2_1_CLK: std_logic := '0';
signal S2_1B_CLK: std_logic := '0';
signal S2_2: std_logic_vector(7 downto 0) := (others => '0');	
signal S2_2_CLK: std_logic := '0';
signal S2_2B_CLK: std_logic := '0';
signal S2_3: std_logic_vector(7 downto 0) := (others => '0');	
signal S2_3_CLK: std_logic := '0';
signal S2_3B_CLK: std_logic := '0';
signal S2_4: std_logic_vector(7 downto 0) := (others => '0');	
signal S2_4_CLK: std_logic := '0';
signal S2_4B_CLK: std_logic := '0';
signal S3_1: std_logic_vector(7 downto 0) := (others => '0');	
signal S3_1_CLK: std_logic := '0';
signal S3_1B_CLK: std_logic := '0';
signal S3_2: std_logic_vector(7 downto 0) := (others => '0');	
signal S3_2_CLK: std_logic := '0';
signal S3_2B_CLK: std_logic := '0';
signal S3_3: std_logic_vector(7 downto 0) := (others => '0');	
signal S3_3_CLK: std_logic := '0';
signal S3_3B_CLK: std_logic := '0';
signal S3_4: std_logic_vector(7 downto 0) := (others => '0');	
signal S3_4_CLK: std_logic := '0';
signal S3_4B_CLK: std_logic := '0';

signal T1: std_logic := '0';
signal T1_CLK: std_logic := '0';
signal T2: std_logic := '0';
signal T2_CLK: std_logic := '0';

signal TRIGGER_REARM_TOGGLE: std_logic:= '0';
signal FORCE_TRIGGER_TOGGLE: std_logic:= '0';
signal START_CAPTURE_TOGGLE: std_logic:= '0';
signal SREG250_READ_TOGGLE: std_logic := '0';

-- DNA Port ------------------------------------
signal DNA_ID_DATA: std_logic_vector(56 downto 0) := (others => '0');
signal DNA_ID_SAMPLE_CLK: std_logic := '0';

--// ARM co-processor interface
signal UC_WEBG: std_logic := '1';  -- WE# through global buffer
signal UC_REBG: std_logic := '1';  -- RE# through global buffer
signal UC_ADDRESS: std_logic_vector(7 downto 0) := (others => '0');
signal UC_AD_LOCAL: std_logic_vector(7 downto 0) := (others => '0');
signal UC_RDN_D: std_logic := '1';
signal UC_RDN_D2: std_logic := '1';
signal CONFIG_CHANGE_TOGGLE: std_logic := '0';
signal CONFIG_CHANGE_PULSE: std_logic := '0';
signal CONFIG_CHANGE_TOGGLE_D: std_logic := '0';
signal CONFIG_CHANGE_TOGGLE_D2: std_logic := '0';
signal CONFIG_CHANGE_PULSE2: std_logic := '0';
signal CONFIG_CHANGE_PULSE2_D: std_logic := '0';
signal CONFIG_CHANGE_TOGGLE2_D: std_logic := '0';
signal CONFIG_CHANGE_TOGGLE2_D2: std_logic := '0';
signal PATH_TO_HOST: integer range 0 to 2 := 0;
signal RX_USB_LAN: std_logic_vector(7 downto 0) := (others => '0'); 

--// Control Registers (initialize here for simulation)
-- REG0-6 reserved for ARM processor
-- The easiest way is to use the ComBlock Control Center GUI to generate correct values for these control registers
-- (or in simulation mode, enter the control register values below)
signal REG7: std_logic_vector(7 downto 0) := x"00"; 	-- 
signal REG8: std_logic_vector(7 downto 0) := x"00"; 	-- 
signal REG9: std_logic_vector(7 downto 0) := x"00"; 	-- 
signal REG10: std_logic_vector(7 downto 0) := x"00";	-- 
signal REG11: std_logic_vector(7 downto 0) := x"00"; 	--  
signal REG12: std_logic_vector(7 downto 0) := x"00"; 	-- 
signal REG13: std_logic_vector(7 downto 0) := x"00"; 	-- 
signal REG14: std_logic_vector(7 downto 0) := x"00"; 	--  
signal REG15: std_logic_vector(7 downto 0) := x"00"; 	-- 
signal REG16: std_logic_vector(7 downto 0) := x"00"; 	-- 
signal REG17: std_logic_vector(7 downto 0) := x"00"; 	--
signal REG18: std_logic_vector(7 downto 0) := x"00"; 	--
signal REG19: std_logic_vector(7 downto 0) := x"00"; 	--
signal REG20: std_logic_vector(7 downto 0) := x"00"; 	--
signal REG21: std_logic_vector(7 downto 0) := x"00"; 	--   
signal REG22: std_logic_vector(7 downto 0) := x"00"; 	--
signal REG23: std_logic_vector(7 downto 0) := x"00"; 	--
signal REG24: std_logic_vector(7 downto 0) := x"00"; 	--
signal REG25: std_logic_vector(7 downto 0) := x"00"; 	--
signal REG26: std_logic_vector(7 downto 0) := x"00"; 	-- 
signal REG27: std_logic_vector(7 downto 0) := x"00"; 	-- 
signal REG28: std_logic_vector(7 downto 0) := x"00"; 	--  
signal REG29: std_logic_vector(7 downto 0) := x"00"; 	-- 
signal REG30: std_logic_vector(7 downto 0) := x"00";	-- 
signal REG31: std_logic_vector(7 downto 0) := x"00";	-- 
signal REG255: std_logic_vector(7 downto 0) := (others => '0');

--// Status Registers
signal SREG6: std_logic_vector(7 downto 0) := (others => '0');
signal SREG7: std_logic_vector(7 downto 0) := (others => '0');
--signal SREG8: std_logic_vector(7 downto 0) := (others => '0');
signal SREG9: std_logic_vector(7 downto 0) := (others => '0');
signal SREG10: std_logic_vector(7 downto 0) := (others => '0');
signal SREG11: std_logic_vector(7 downto 0) := (others => '0');
signal SREG12: std_logic_vector(7 downto 0) := (others => '0');
signal SREG13: std_logic_vector(7 downto 0) := (others => '0');
signal SREG14: std_logic_vector(7 downto 0) := (others => '0');
signal SREG15: std_logic_vector(7 downto 0) := (others => '0');
signal SREG16: std_logic_vector(7 downto 0) := (others => '0');
signal SREG17: std_logic_vector(7 downto 0) := (others => '0');
signal SREG18: std_logic_vector(7 downto 0) := (others => '0');
signal SREG19: std_logic_vector(7 downto 0) := (others => '0');
signal SREG20: std_logic_vector(7 downto 0) := (others => '0');
signal SREG21: std_logic_vector(7 downto 0) := (others => '0');
signal SREG22: std_logic_vector(7 downto 0) := (others => '0');
signal SREG23: std_logic_vector(7 downto 0) := (others => '0');
signal SREG24: std_logic_vector(7 downto 0) := (others => '0');
signal SREG25: std_logic_vector(7 downto 0) := (others => '0');
signal SREG26: std_logic_vector(7 downto 0) := (others => '0');
signal SREG27: std_logic_vector(7 downto 0) := (others => '0');
signal SREG28: std_logic_vector(7 downto 0) := (others => '0');
signal SREG8_READ_TOGGLE: std_logic := '0';
signal SREG8_READ_TOGGLE_D: std_logic := '0';
signal SREG8_READ_TOGGLE_D2: std_logic := '0';
signal LATCH_MONITORING_PULSE: std_logic := '0';
signal SREG254_READ_TOGGLE: std_logic := '0';			

--// ComScope control registers
signal REG237: std_logic_vector(7 downto 0):= "00000000";
signal REG238: std_logic_vector(7 downto 0):= "00000000";
signal REG239: std_logic_vector(7 downto 0):= "00000000";
signal REG240: std_logic_vector(7 downto 0):= "00001101";	-- Trace 1 signal selection
signal REG241: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG242: std_logic_vector(7 downto 0):= "00001110";	-- Trace 2 signal selection
signal REG243: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG244: std_logic_vector(7 downto 0):= "00000000";	-- Trace 3 signal selection
signal REG245: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG246: std_logic_vector(7 downto 0):= "00000000";	-- Trace 4 signal selection
signal REG247: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG248: std_logic_vector(7 downto 0):= "10001101";	-- Trigger edge, trigger signal
signal REG249: std_logic_vector(7 downto 0):= "10111111";	-- Trigger threshold
signal REG250: std_logic_vector(7 downto 0):= "00000000";
signal REG251: std_logic_vector(7 downto 0):= "00000000";


--// FPGA option and revision (ASCII format)
-- These two ASCII characters appear in the Comblock Control Center.
-- Very helpful in identifying which firmware version is currently running
-- in the FPGA. 
-- '0' = 48 decimal = x"30"
-- 'A' = 65 decimal = x"41"
--constant OPTION: std_logic_vector(7 downto 0) := x"41";	--A hardware option for complex baseband input, 26 MHz TCXO freq. ref	
constant OPTION: std_logic_vector(7 downto 0) := x"42";	 	--B hardware option for 70 MHz IF undersampling, 26 MHz TCXO freq. ref
--constant OPTION: std_logic_vector(7 downto 0) := x"43";	--C hardware option for 140 MHz IF undersampling, 26 MHz TCXO freq. ref
--constant OPTION: std_logic_vector(7 downto 0) := x"44";	--D hardware option for complex baseband input, 10 MHz ext freq. ref (3rd harmonic)
--constant OPTION: std_logic_vector(7 downto 0) := x"45";	--E hardware option for 70 MHz IF undersampling, 10 MHz ext freq. ref (3rd harmonic)
--constant OPTION: std_logic_vector(7 downto 0) := x"46";	--F hardware option for 140 MHz IF undersampling, 10 MHz ext freq. ref (3rd harmonic)
constant REVISION: std_logic_vector(7 downto 0) := x"31";	-- '1'
constant SIMULATION: std_logic := '0';	-- set to '1' during simulation

-- dependent constants
function get_REFCLK_FREQUENCY(OPTION: std_logic_vector) return integer is
variable result : integer;
begin
	if(OPTION = x"41") or (OPTION = x"42") or (OPTION = x"43") then
		result := 26;	-- 26 MHz TCXO
	elsif(OPTION = x"44") or (OPTION = x"45") or (OPTION = x"46") then
		result := 30;	-- 10MHz external frequency reference x3rd harmonic
	end if;
return result;
end function;

function get_CLK_FREQUENCY(OPTION: std_logic_vector) return integer is
variable result : integer;
begin
	if(OPTION = x"42") or (OPTION = x"45") then
		result := 56;	-- 56 MHz for IF undersampling at 70 MHz (=56*1.25)
	else
		result := 80;	-- 80 MHz for IF undersampling at 140 MHz (80*1.75) or baseband
	end if;
return result;
end function;
function get_PLL_MULT(OPTION: std_logic_vector) return integer is
variable result : integer;
begin
	if(OPTION = x"42") then
		result := 28;	-- 728 MHz VCO / 28 = 26 MHz input reference freq.
	elsif(OPTION = x"41") or (OPTION = x"43") then
		result := 37;	-- 962 MHz VCO / 37 = 26 MHz input reference freq.
	elsif(OPTION = x"45") then
		result := 28;	-- 840 MHz VCO / 28 = 30 MHz input reference freq. (after 3rd harmonic is filtered)
	elsif(OPTION = x"44") or (OPTION = x"46") then
		result := 24;	-- 720 MHz VCO / 24 = 30 MHz input reference freq. (after 3rd harmonic is filtered)
	end if;
return result;
end function;
function get_PLL_DIVIDE(OPTION: std_logic_vector) return integer is
variable result : integer;
begin
	if(OPTION = x"42") then
		result := 13;	-- 728 MHz VCO / 13 = 56 MHz sampling frequency.
	elsif(OPTION = x"41") or (OPTION = x"43") then
		result := 12;	-- 962 MHz VCO / 12 = 80.1666 MHz sampling frequency
	elsif(OPTION = x"45") then
		result := 15;	-- 840 MHz VCO / 15 = 56 MHz sampling frequency.
	elsif(OPTION = x"44") or (OPTION = x"46") then
		result := 9;	-- 720 MHz VCO / 9 = 80 MHz sampling frequency.
	end if;
return result;
end function;

constant REFCLK_FREQUENCY: integer := get_REFCLK_FREQUENCY(OPTION);	-- frequency reference = 26 or 10 MHz depending on the hardware option
constant CLK_FREQUENCY: integer := get_CLK_FREQUENCY(OPTION);	-- fclk_p = 56 or 80 MHz depending on the hardware option
constant CLK_ADC_PLL_MULT: integer := get_PLL_MULT(OPTION);
constant CLK_ADC_PLL_DIVIDE: integer := get_PLL_DIVIDE(OPTION);

--------------------------------------------------------
--      IMPLEMENTATION
--------------------------------------------------------
begin
------------------------------------------------------------------------------------------------------------------------
--// CLOCKS, RESET 
------------------------------------------------------------------------------------------------------------------------
-- higher-stability frequency reference either 26 MHz TCXO or 10 MHz external clock through J5.
-- Depending on the hardware platform configuration (R20/R21)

-- 10 MHz in ----------------------------------------
-- TODO

-- 26 MHz in ----------------------------------------
BUFG_CLKREF: BUFG port map(I => CLKREF, O => CLKREFg);

-- ADC OUTPUT clock generation. 
-- ADC sampling rate and processing clock CLK_P frequency depend on the hardware option:
-- 80 MSamples/s for hardware option -A and -C (baseband or 140 MHz IF), 56 MHz for option -B (70 MHz IF)
-- This clock ONLY goes to the ADCs, which, in turn, generate ADCx_DCO synchronous with ADC samples, 
-- which we convert to processing clock (CLK_P).
-- Important: PLL_BASE has much lower jitter than DCM_CLKGEN
PLL_BASE_001 : PLL_BASE
generic map
(BANDWIDTH            => "LOW",	-- fixed PLL, prefer low jitter to fast lock
 CLK_FEEDBACK         => "CLKFBOUT",
 COMPENSATION         => "INTERNAL",
 DIVCLK_DIVIDE        => 1,	
 CLKFBOUT_MULT        => CLK_ADC_PLL_MULT,	-- 728/962 MHz VCO / CLK_ADC_PLL_MULT = 26 MHz input reference freq.
 CLKFBOUT_PHASE       => 0.000,
 CLKOUT0_DIVIDE       => CLK_ADC_PLL_DIVIDE,	-- 56/80.1666 MHz output
 CLKOUT0_PHASE        => 0.0,
 CLKOUT0_DUTY_CYCLE   => 0.500,
 CLKIN_PERIOD         => 38.461,
 REF_JITTER           => 0.010)
port map
 -- Output clocks
(CLKFBOUT            => ADC_CLK_CLKFB1,
 CLKOUT0             => CLK_ADCg,
 -- Status and control signals
 LOCKED              => PLL1_LOCKED,
 RST                 => '0',
 -- Input clock control
 CLKFBIN             => ADC_CLK_CLKFB1,
 CLKIN               => CLKREFg	-- 26 MHz external frequency reference
 );
CLK_ADCg_N <= not CLK_ADCg;

-- send ADC clock out
-- ADC clock output
-- It is recommended to use a Clock Forwarding technique to create a reliable and repeatable 
-- low skew solution, i.e. use ODDR2 to regenerate the output clock.
ADC_CLKOUT_GEN_001P : ODDR2
generic map(
	DDR_ALIGNMENT => "NONE", -- Sets output alignment to "NONE", "C0", "C1" 
	INIT => '0', -- Sets initial state of the Q output to '0' or '1'
	SRTYPE => "SYNC") -- Specifies "SYNC" or "ASYNC" set/reset
port map (
	Q => ADC_CLK, -- 1-bit output data
	C0 => CLK_ADCg, -- 1-bit clock input
	C1 => CLK_ADCg_N, -- 1-bit clock input
	CE => '1',  -- 1-bit clock enable input
	D0 => '1',   -- 1-bit data input (associated with C0)
	D1 => '0',   -- 1-bit data input (associated with C1)
	R => '0',    -- 1-bit reset input
	S => '0'     -- 1-bit set input
);

-- LVDS output
OBUFDS_inst : OBUFDS
generic map (
	IOSTANDARD => "LVDS_33")
port map (
	O => ADC_CLKP,     -- Diff_p output (connect directly to top-level port)
	OB => ADC_CLKN,   -- Diff_n output (connect directly to top-level port)
	I => ADC_CLK      -- Buffer input 
);

--// Make ADC_DCO(0) a global clock = CLK_P
-- Key assumption: not much skew between ADC_DCO(0) and ADC_DCO(1).
-- Select best edge to resample ADC data(rising or falling?) Inversion by selecting CLKFX180 or CLKFX above.
BUFG_ADC1_DCO: BUFG port map(I => ADC1_DCO, O => CLK_P);

-- USB clock generation. Here 60 MHz. from 26MHz ref clock
-- use PLL_BASE for lower phase noise than DCM_CLKGEN
PLL_BASE_002 : PLL_BASE
generic map
(BANDWIDTH            => "LOW",	-- fixed PLL, prefer low jitter to fast lock
 CLK_FEEDBACK         => "CLKFBOUT",
 COMPENSATION         => "INTERNAL",
 DIVCLK_DIVIDE        => 1,	
 CLKFBOUT_MULT        => 30,	-- 780 MHz VCO / 30 = 26 MHz input reference freq.
 CLKFBOUT_PHASE       => 0.000,
 CLKOUT0_DIVIDE       => 13,	-- 60 MHz output
 CLKOUT0_PHASE        => 0.0,
 CLKOUT0_DUTY_CYCLE   => 0.500,
 CLKIN_PERIOD         => 38.461,
 REF_JITTER           => 0.010)
port map
 -- Output clocks
(CLKFBOUT            => CLK60_CLKFB1,
 CLKOUT0             => CLK60,
 -- Status and control signals
 LOCKED              => PLL2_LOCKED,
 RST                 => '0',
 -- Input clock control
 CLKFBIN             => CLK60_CLKFB1,
 CLKIN               => CLKREFg	-- 26 MHz external frequency reference
 );
BUFG_USB_CLK: BUFG port map(I => CLK60, O => CLK60G);

-- Generate miscellaneous global clocks and resets from the 26 MHz reference clock
RESET_STATE_MACHINE_001: process(CLKREFg)
begin
	if rising_edge(CLKREFg) then
		-- PLL need time to lock. The reset of the application is still in reset
		if(RESET_COUNTER(15) = '0') and (SIMULATION = '0') then
			-- wait a bit before asserting the PLL lock
			RESET_COUNTER <= RESET_COUNTER + 1;
		elsif(RESET_COUNTER(5) = '0') and (SIMULATION = '1') then  
			-- make it short during simulations
			-- wait a bit before asserting the PLL lock
			RESET_COUNTER <= RESET_COUNTER + 1;
		elsif(PLL1_LOCKED = '1') and (PLL2_LOCKED = '1') and (PLL_LOCKED_TX = '1') then
			-- assert all PLLs and DCMs lock statuses
			-- Note: since these signals are generated with several different clocks, use the timing ignore TIG command
			-- in the .ucf file
			ASYNC_RESETN <= '1';
		end if;
	end if;
end process;

ASYNC_RESET <= not ASYNC_RESETN;

-- generate a reset synchronous with CLK_P
SYNC_RESET_CLK_P_GEN: process(ASYNC_RESET, CLK_P)
begin
	if(ASYNC_RESET = '1') then
		SYNC_RESET_CLK_P <= '1';
	elsif rising_edge(CLK_P) then
		ASYNC_RESET_D <= ASYNC_RESET;
		
		if(ASYNC_RESET_D = '1') then
			SYNC_RESET_CLK_P <= '1';
		else
			SYNC_RESET_CLK_P <= '0';
		end if;
	end if;
end process;

-- generate a reset synchronous with CLK_DACg
SYNC_RESET_CLK_DAC_GEN: process(ASYNC_RESET, CLK_DACg)
begin
	if(ASYNC_RESET = '1') then
		SYNC_RESET_CLK_DAC <= '1';
	elsif rising_edge(CLK_DACg) then
		ASYNC_RESET2_D <= ASYNC_RESET;
		
		if(ASYNC_RESET2_D = '1') then
			SYNC_RESET_CLK_DAC <= '1';
		else
			SYNC_RESET_CLK_DAC <= '0';
		end if;
	end if;
end process;

	-- external input clock
BUFG_006: BUFG  port map (I=> UC_WEB_IN, O=> UC_WEBG);	
	-- ARM WE#
BUFG_007: BUFG  port map (I=> UC_REB_IN, O=> UC_REBG);	
	-- ARM RE#


-- DAC SAMPLING CLOCK ------------------------------------------------
-- generate the DAC sampling clock CLK from a 200+ MHz PLL
-- Output range 45-90 MHz (DAC sampling clock)
-- The 90 MHz maximum DAC sampling rate is limited by the nearly full packing of the FPGA (78% LUTs) combined with the 
-- somewhat slower L1 FPGA speed grade
DAC_CLK_FREQ <= REG22(3 downto 0) & REG21 & REG20;		

HIGH_SPEED_NCO_001: HIGH_SPEED_NCO 
GENERIC MAP(
	REFCLK_FREQUENCY => REFCLK_FREQUENCY
)
PORT MAP(
	REFCLK => CLKREFg,
	NCO_FREQ => DAC_CLK_FREQ,
	CLK_OUTG => CLK_DACg,
	LOCKED_PLL => PLL_LOCKED_TX
);

-- add BUFG
--BUFG_008: BUFG port map(I => CLK_DAC, O => CLK_DACg);


--// DACs ------------------------------------------------------------------------------
-- forward DAC sampling clock. Invert because DAC reads data at the rising edge.
DAC_CLK_GEN_001 : ODDR2
generic map(
	DDR_ALIGNMENT => "NONE", -- Sets output alignment to "NONE", "C0", "C1" 
	INIT => '0', -- Sets initial state of the Q output to '0' or '1'
	SRTYPE => "SYNC") -- Specifies "SYNC" or "ASYNC" set/reset
port map (
	Q => DAC_CLKP, -- 1-bit output data
	C0 => CLK_DACg, -- 1-bit clock input
	C1 => not CLK_DACg, -- 1-bit clock input
	CE => '1',  -- 1-bit clock enable input
	D0 => '0',   -- 1-bit data input (associated with C0)
	D1 => '1',   -- 1-bit data input (associated with C1)
	R => '0',    -- 1-bit reset input
	S => '0'     -- 1-bit set input
);

-- 1.8V fake differential
DAC_CLK_GEN_002 : ODDR2
generic map(
	DDR_ALIGNMENT => "NONE", -- Sets output alignment to "NONE", "C0", "C1" 
	INIT => '0', -- Sets initial state of the Q output to '0' or '1'
	SRTYPE => "SYNC") -- Specifies "SYNC" or "ASYNC" set/reset
port map (
	Q => DAC_CLKN, -- 1-bit output data
	C0 => CLK_DACg, -- 1-bit clock input
	C1 => not CLK_DACg, -- 1-bit clock input
	CE => '1',  -- 1-bit clock enable input
	D0 => '1',   -- 1-bit data input (associated with C0)
	D1 => '0',   -- 1-bit data input (associated with C1)
	R => '0',    -- 1-bit reset input
	S => '0'     -- 1-bit set input
);

-- default format is unsigned binary
-- reclock to place FF in IOB
MOD_DATA_OUT: process(CLK_DACg)
begin
	if rising_edge(CLK_DACg) then
		DAC1_D(11) <= not DAC1(11);
		DAC1_D(10 downto 0) <= DAC1(10 downto 0);
		DAC2_D(11) <= not DAC2(11);
		DAC2_D(10 downto 0) <= DAC2(10 downto 0);
	end if;
end process;

DAC_SPI(3) <= '1';	-- pin mode
DAC_SPI(1) <= '1';	-- differential clock 
DAC_SPI(2) <= '0';	-- straight binary format
DAC1_SPI_CSB <= '0';	-- no sleep
DAC2_SPI_CSB <= '0';	-- no sleep

-- debug code
-- hardware check: read the AD9706 reg0x0. Expecting x80
--DAC_SPI(3) <= '0';	-- SPI mode
--DAC2_SPI_CSB <= '0';	-- no sleep
--SPI_001: SPI 
--generic map(
--	NDIV => 8)	-- 20 MHz max SPI clk
--PORT MAP (
--		 CLK => CLK_DACg,
--		 ASYNC_RESET => '0',
--		 ADDR => "00000",	
--		 DATA_OUT => x"00",
--		 WRITE_REQ => '0',
--		 READ_REQ => CONFIG_CHANGE_PULSE2,
--		 DATA_IN => SREG10,
--		 DATA_IN_CLK => open,
--		 BUSY => open,
--		 CSB => DAC1_SPI_CSB,
--		 SCLK => DAC_SPI(1),
--		 SDIO => DAC_SPI(2)
--	  );



--// EXTERNAL TRANSCEIVER CONTROL -------------------------
-- binary controls on 40 pin connector
EXT_TRANSCEIVER_001: process(CLK_P)
begin
	if rising_edge(CLK_P) then
--		D_CNTRL(2) <= ;
--		D_CNTRL(3) <= ;
	end if;
end process;

--//==================================================================================
--// START OF USER CODE
--//==================================================================================
-- Place user code here
-- 1. Output samples are in DAC1,DAC2 (12-bit, 2's complement format)


--//==================================================================================
--// END OF USER CODE
--//==================================================================================

--// ADCs ---------------------------------------------------------
-- Immediately reclock samples in IOBs
-- convert to unsigned binary to 2's complement. Extend precision to 14 bits.
ADC_DATA_IN: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		ADC_DATA_I_IN(13) <= not ADC1_D(9);
		ADC_DATA_I_IN(12 downto 4) <= ADC1_D(8 downto 0);
		ADC_DATA_I_IN(3 downto 0) <= "0000";
		ADC_DATA_Q_IN(13) <= not ADC2_D(9);
		ADC_DATA_Q_IN(12 downto 4) <= ADC2_D(8 downto 0);
		ADC_DATA_Q_IN(3 downto 0) <= "0000";
	end if;
end process;

----	AUXDAC1_DIN => AGC_DAC,
--

--// USB INTERFACE ---------------------
-- The USB component is available in .ngc form (not source code).
-- Two virtual channels are available:
-- Channel 1 for Monitoring and Control. PART OF THE FRAMEWORK. DO NOT CHANGE.
-- Channel 2 is a payload channel (available for custom applications). 
-- Tie the channel 2 inputs and outputs to the application input and 
-- output streams as needed.

USB20ULPI_001: USB20ULPI_CLK2PHY port map(
	USB_CLK60G => CLK60G,
	ASYNC_RESET => SYNC_RESET_CLK_P,
	CLK_P => CLK_P,

	-- interface with USB PHY. Direct connection to I/O pins.
	USB_ULPI_DATA => USB_ULPI_DATA,
	USB_ULPI_STP => USB_ULPI_STP_LOCAL,
	USB_ULPI_NXT => USB_ULPI_NXT,
	USB_ULPI_DIR => USB_ULPI_DIR,

	-- M&C channel. Interfaces with ARM uC. Uses data stream 1
	DATA1_OUT => RX_USB,
	DATA1_OUT_SAMPLE_CLK => open,
	DATA1_OUT_BUFFER_EMPTY => RX_USB_BUFFER_EMPTY,
	DATA1_OUT_SAMPLE_CLK_REQ => RX_USB_SAMPLE_CLK_REQ,
	DATA1_IN => TX_USB,
   DATA1_IN_SAMPLE_CLK => TX_USB_SAMPLE_CLK,
	DATA1_IN_SAMPLE_CLK_REQ => open,	
		-- ARM uC is much slower than USB. no need to verify tx flow control.

	-- Payload channel. Data stream 2
	DATA2_OUT => open,	-- user to read USB payload data here
	DATA2_OUT_SAMPLE_CLK => open,
	DATA2_OUT_BUFFER_EMPTY => open,
	DATA2_OUT_SAMPLE_CLK_REQ => '1',
	DATA2_IN => x"00",	-- user to send USB payload data here
	DATA2_IN_SAMPLE_CLK => '0',
	DATA2_IN_SAMPLE_CLK_REQ => open,	-- user to check USB tx flow control here before sending payload data

	PHY_VENDOR_ID => USB_PHY_VENDOR_ID,
	CS1 => open,
	CS1_CLK => open,
	CS2 => open,
	CS2_CLK => open,
	CS3 => open,
	CS3_CLK => open,
	T1 => open,
	T1_CLK => open,
	T2 => open,
	T2_CLK => open,
	USB_TP => open
);
USB_ULPI_STP <= USB_ULPI_STP_LOCAL;
USB_RESETN <= not ASYNC_RESET;

-- hardware self-check: report USB PHY ID
SREG6 <= USB_PHY_VENDOR_ID;

-- send 60 MHz clock to USB PHY
-- It is recommended to use a Clock Forwarding technique to create a reliable and repeatable 
-- low skew solution, i.e. use ODDR2 to regenerate the output clock.
--  + invert DAC clock to create a delay between output data 
USB20ULPI_002 : ODDR2
generic map(
	DDR_ALIGNMENT => "NONE", -- Sets output alignment to "NONE", "C0", "C1" 
	INIT => '0', -- Sets initial state of the Q output to '0' or '1'
	SRTYPE => "SYNC") -- Specifies "SYNC" or "ASYNC" set/reset
port map (
	Q => USB_CLKREF60, -- 1-bit output data
	C0 => CLK60G, -- 1-bit clock input
	C1 => not CLK60G, -- 1-bit clock input
	CE => '1',  -- 1-bit clock enable input
	D0 => '0',   -- 1-bit data input (associated with C0)
	D1 => '1',   -- 1-bit data input (associated with C1)
	R => '0',    -- 1-bit reset input
	S => '0'     -- 1-bit set input
);

--//-- 10/100/1000 Mbps Ethernet  -------
ETHERNET_001: if (SIMULATION = '0') generate
	-- Interface with COM-5401 PHY#1 or COM-5102 PHY on the left connector. Also skip for faster simulations.
	CLK_SKEW <= "1111111111";  -- tested successfully with x3FF
	-- new: MAC address automatically generated from unique DNA_ID
	LAN1_MAC_ADDR_GEN: process(CLK_P)
	begin
		if rising_edge(CLK_P) then
			if(DNA_ID_SAMPLE_CLK = '1') then
					LAN1_MAC_ADDR <= x"00" & DNA_ID_DATA(39 downto 0);	-- unique, unicast
			end if;
		end if;
	end process;
	-- User TODO: Configure the LAN/IP here:
	RECLOCK_CTRLS_006: process(CLK_P)
	begin
		if rising_edge(CLK_P) then
			IP1_ADDRv4(31 downto 0) <= x"AC100180";	-- user-defined local IP address (here 172.16.1.128)
			IP1_SUBNET_MASK <= x"FFFFFF00";
			IP1_GATEWAY <= x"AC100103";
		end if;
	end process;

	LAN_001: COM5401 
	GENERIC MAP(
		MII_SEL => '0',   	-- select which MII interface: 0 = RGMII, 1 = GMII/MII
		PHY_ADDR => "00111",	-- PHY_AD0/1 pulled-up by 1KOhm, PHY_AD2 pulled-up in .ucf file.
		CLK_FREQUENCY => CLK_FREQUENCY,	-- CLK_P in MHz, as time reference
		SIMULATION => SIMULATION
	)
	PORT MAP(
		CLK => CLK_P,
		SYNC_RESET => SYNC_RESET_CLK_P,
		MAC_TX_CONFIG => X"0003",	-- MAC must must provide pad + crc32
		MAC_RX_CONFIG => x"000F",	-- promiscuous mode, strip crc32, accept broadcast/multicast
		MAC_ADDR => LAN1_MAC_ADDR,
		PHY_CONFIG_CHANGE => CONFIG_CHANGE_PULSE,	
		PHY_RESET => '0',
		SPEED => "11",	-- supersedes defaults within if PHY_CONFIG_CHANGE = '1'
		DUPLEX => '1',	-- supersedes defaults within if PHY_CONFIG_CHANGE = '1'
		TEST_MODE => "00",	-- supersedes defaults within if PHY_CONFIG_CHANGE = '1'
		POWER_DOWN => '0', -- supersedes defaults within if PHY_CONFIG_CHANGE = '1'
		CLK_SKEW => CLK_SKEW, -- supersedes defaults within if PHY_CONFIG_CHANGE = '1'
		MAC_TX_DATA => LAN1_MAC_TX_DATA,
		MAC_TX_DATA_VALID => LAN1_MAC_TX_DATA_VALID,
		MAC_TX_EOF => LAN1_MAC_TX_EOF,
		MAC_TX_CTS => LAN1_MAC_TX_CTS,
		MAC_RX_DATA => LAN1_MAC_RX_DATA,	
		MAC_RX_DATA_VALID => LAN1_MAC_RX_DATA_VALID,
		MAC_RX_SOF => LAN1_MAC_RX_SOF,
		MAC_RX_EOF => LAN1_MAC_RX_EOF,
		MAC_RX_CTS => '1',  -- follow-on processing is expected to always accept data even at max speed.
		-- RGMII interface ----------------
		TXC => LAN_TXC,	
		TXD => LAN_TXD,
		TX_CTL => LAN_TX_CTL,
		RXC => LAN_RXC,
		RXD => LAN_RXD,
		RX_CTL => LAN_RX_CTL,
		RESET_N => LAN_RESET_N,
		MCLK => LAN_MCLK,
		MDIO => LAN_MDIO,	-- PHY#1
		-- end of RGMII interface --------
		MII_TX_CLK => '0',	-- MII interface is unused -----
		GMII_TX_CLK => open,
		GMII_MII_TXD => open,
		GMII_MII_TX_EN => open,
		GMII_MII_TX_ER => open,
		GMII_MII_CRS => '0',
		GMII_MII_COL => '0',
		GMII_MII_RX_CLK => '0',
		GMII_MII_RXD => x"00",
		GMII_MII_RX_DV => '0',
		GMII_MII_RX_ER => '0',  -- end of MII interface ------
		LINK_STATUS => LAN1_LINK_STATUS,
		SPEED_STATUS => LAN1_SPEED_STATUS,
		DUPLEX_STATUS => LAN1_DUPLEX_STATUS,
		PHY_ID => LAN1_PHY_ID,
		N_RX_FRAMES => open,
		N_RX_BAD_CRCS => open,
		N_RX_FRAMES_TOO_SHORT => open,
		N_RX_FRAMES_TOO_LONG => open,
		N_RX_WRONG_ADDR => open,
		N_RX_LENGTH_ERRORS => open,
		CS_CLKG => open,
		CS1 => open,
		CS1_CLK => open,
		CS2 => open,
		CS2_CLK => open,
		TP => COM5401_TP
	);


	-- hardware test: read PHY_ID
	SREG7 <= LAN1_PHY_ID(7 downto 0);
	--SREGXX <= LAN1_PHY_ID(15 downto 8);

	-- enabled: 2 TCP (port 1028, reserved for M&C, port 1024 for user-defined data transfer), 1 UDP tx/rx(port TBD below)
	IPSTACK_001: COM5402 
	GENERIC MAP(
		NUDPTX => 1,
		NUDPRX => 1,
		NTCPSTREAMS => 2,  
		CLK_FREQUENCY => 80,	
		TX_IDLE_TIMEOUT => 50,	-- 50*4us = 200us 
		SIMULATION => SIMULATION
	)
	PORT MAP(
		CLK => CLK_P,
		ASYNC_RESET => SYNC_RESET_CLK_P,	-- to be progressively phased out. Replace with SYNC_RESET
		SYNC_RESET => SYNC_RESET_CLK_P,
		MAC_ADDR => LAN1_MAC_ADDR,
		IPv4_ADDR => IP1_ADDRv4,
		IPv6_ADDR => x"00000000000000000000000000000000",
	   MULTICAST_IP_ADDR => x"00000000",  -- no multicast
		SUBNET_MASK => IP1_SUBNET_MASK,
		GATEWAY_IP_ADDR => IP1_GATEWAY,
		CONNECTION_RESET => "00",
		-- MAC interface. Connect to COM5401 Ethernet MAC
		MAC_TX_DATA => LAN1_MAC_TX_DATA,
		MAC_TX_DATA_VALID => LAN1_MAC_TX_DATA_VALID,
		MAC_TX_SOF => LAN1_MAC_TX_SOF,
		MAC_TX_EOF => LAN1_MAC_TX_EOF,
		MAC_TX_CTS => LAN1_MAC_TX_CTS,
		MAC_RX_DATA => LAN1_MAC_RX_DATA,
		MAC_RX_DATA_VALID => LAN1_MAC_RX_DATA_VALID,
		MAC_RX_SOF => LAN1_MAC_RX_SOF,
		MAC_RX_EOF => LAN1_MAC_RX_EOF,
		-- UDP rx
		UDP_RX_DATA => open,
		UDP_RX_DATA_VALID => open,
		UDP_RX_SOF => open,
		UDP_RX_EOF => open,
		UDP_RX_DEST_PORT_NO => x"0000",
		-- UDP tx
		UDP_TX_DATA => x"00",
		UDP_TX_DATA_VALID => '0',
		UDP_TX_SOF => '0',
		UDP_TX_EOF => '0',
		UDP_TX_CTS => open,
		UDP_TX_ACK => open,
		UDP_TX_NAK => open,
		UDP_TX_DEST_IP_ADDR => x"00000000000000000000000000000000",
		UDP_TX_DEST_PORT_NO => x"0000",	-- TBD
		UDP_TX_SOURCE_PORT_NO => x"0000",	-- TBD
		-- TCP rx streams
		TCP_RX_DATA => LAN1_TCP_RX_DATA,
		TCP_RX_DATA_VALID => LAN1_TCP_RX_DATA_VALID,
		TCP_RX_RTS => open,
		TCP_RX_CTS => LAN1_TCP_RX_CTS,	
		-- TCP tx streams
		TCP_TX_DATA => LAN1_TCP_TX_DATA,
		TCP_TX_DATA_VALID => LAN1_TCP_TX_DATA_VALID,
		TCP_TX_CTS => LAN1_TCP_TX_CTS,	
		-- Monitoring
		TCP_CONNECTED_FLAG => TCP_CONNECTED_FLAG,
	--	CS1 => S1_1,
	--	CS1_CLK => S1_1_CLK,
	--	CS2 => S2_1,
	--	CS2_CLK => S2_1_CLK,
		TP => COM5402_TP	
	);

	-- one elastic buffer for each tcp rx stream
	-- M&C, TCP port 1028 (port is defined within COM5402 component)
	ELASTIC_BUFFER_NRAMB_0: ELASTIC_BUFFER_NRAMB 
	generic map(
		NRAMB => 1,
		NBITS => 9,
		READ_AHEAD_1ADDRESS => '1'	-- no read latency, tricky timing ok because ARM is slower.
	)
	PORT MAP(
		CLK => CLK_P,
		SYNC_RESET => SYNC_RESET_CLK_P,
		DATA_IN => "0" & LAN1_TCP_RX_DATA(0),	-- 9 bit wide
		SAMPLE_CLK_IN => LAN1_TCP_RX_DATA_VALID(0),
		SAMPLE_CLK_IN_REQ => LAN1_TCP_RX_CTS(0),
		DATA_OUT => LAN_RX_DATA1_X,	-- 9 bit wide
		SAMPLE_CLK_OUT => open,
		SAMPLE_CLK_OUT_E => open,
		SAMPLE_CLK_OUT_REQ => RX_LAN_SAMPLE_CLK_REQ,
		BUFFER_EMPTY => RX_LAN_BUFFER_EMPTY
	);
	RX_LAN <= LAN_RX_DATA1_X(7 downto 0);


	-- user data, TCP port 1024 (port is defined within COM5402 component)
	ELASTIC_BUFFER_NRAMB_1: ELASTIC_BUFFER_NRAMB 
	generic map(
		NRAMB => 1,
		NBITS => 9,
		READ_AHEAD_1ADDRESS => '0'	
	)
	PORT MAP(
		CLK => CLK_P,
		SYNC_RESET => SYNC_RESET_CLK_P,
		DATA_IN => "0" & LAN1_TCP_RX_DATA(1),	-- 9 bit wide
		SAMPLE_CLK_IN => LAN1_TCP_RX_DATA_VALID(1),
		SAMPLE_CLK_IN_REQ => LAN1_TCP_RX_CTS(1),
		DATA_OUT => open,	-- 9 bit wide		-- User to receive TCP data here
		SAMPLE_CLK_OUT => open,
		SAMPLE_CLK_OUT_E => open,
		SAMPLE_CLK_OUT_REQ => '1',
		BUFFER_EMPTY => open
	);

end generate;

--// COMSCOPE -----------------------------------------------------
COMSCOPE_GEN: if (SIMULATION = '0') generate
	--  Comment in/out as needed. unused during simulation
	-- 
	-- Define Comscope traces here 
	-- Trace 1
--	S1_1 <= DATA1I(12 downto 5);	-- receiver1 output 
--	S1_1_CLK <= SAMPLE1_CLK;
--
--	-- Trace 2
--	S2_1 <= DATA1Q(12 downto 5);	-- receiver1 output 
--	S2_1_CLK <= SAMPLE1_CLK;
--	S2_3 <= RX_DATA1_I(13 downto 6);	-- receiver1 input I
--	S2_3_CLK <= '1';
--
--	-- Trace 3
--	S3_1 <= RX_DATA1_Q(13 downto 6);	-- receiver1 input Q
--	S3_1_CLK <= '1';
--	
--	-- Trigger (1-bit format, undefine
--	T1 <= '0';	
--	T1_CLK <= '0';

	S1_1B_CLK <= S1_1_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock
	S1_2B_CLK <= S1_2_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock
	S1_3B_CLK <= S1_3_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock
	S1_4B_CLK <= S1_4_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock

	S2_1B_CLK <= S2_1_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock
	S2_2B_CLK <= S2_2_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock
	S2_3B_CLK <= S2_3_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock
	S2_4B_CLK <= S2_4_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock

	S3_1B_CLK <= S3_1_CLK or REG244(7);	-- Nominal sampling clock or real-time processing clock
	S3_2B_CLK <= S3_2_CLK or REG244(7);	-- Nominal sampling clock or real-time processing clock
	S3_3B_CLK <= S3_3_CLK or REG244(7);	-- Nominal sampling clock or real-time processing clock
	S3_4B_CLK <= S3_4_CLK or REG244(7);	-- Nominal sampling clock or real-time processing clock


	COMSCOPE_001: COMSCOPE 
	generic map(
		TRACE1_ENABLED => '1',
		TRACE2_ENABLED => '1',
		TRACE3_ENABLED => '1',
		TRACE4_ENABLED => '0'
	)
	port map(
		CLK => CLK_P,
		ASYNC_RESET => ASYNC_RESET,
		SIGNED_REPRESENTATION_U_SIGNED_N => REG248(6),	-- trigger format signed(1) or unsigned(0)
		REG237 => REG237,
		REG238 => REG238,
		REG239 => REG239,
		REG240 => REG240,
		REG241 => REG241,
		REG242 => REG242,
		REG243 => REG243,
		REG244 => REG244,
		REG245 => REG245,
		REG246 => REG246,
		REG247 => REG247,
		REG248 => REG248,
		REG249 => REG249,
		SIGNAL_1_BIT_1_1 => '0',
		SIGNAL_1_BIT_SAMPLE_CLK_1_1 => '0',
		SIGNAL_1_BIT_1_2 => '0',
		SIGNAL_1_BIT_SAMPLE_CLK_1_2 => '0',
		SIGNAL_1_BIT_1_3 => '0',
		SIGNAL_1_BIT_SAMPLE_CLK_1_3 => '0',
		SIGNAL_1_BIT_1_4 => '0',
		SIGNAL_1_BIT_SAMPLE_CLK_1_4 => '0',
	--	SIGNAL_2_BIT_1_1 => (others => '0'),
	--	SIGNAL_2_BIT_SAMPLE_CLK_1_1 => '0',
	--	SIGNAL_2_BIT_1_2 => (others => '0'),
	--	SIGNAL_2_BIT_SAMPLE_CLK_1_2 => '0',
	--	SIGNAL_2_BIT_1_3 => (others => '0'),
	--	SIGNAL_2_BIT_SAMPLE_CLK_1_3 => '0',
	--	SIGNAL_2_BIT_1_4 => (others => '0'),
	--	SIGNAL_2_BIT_SAMPLE_CLK_1_4 => '0',
	--	SIGNAL_4_BIT_1_1 => (others => '0'),
	--	SIGNAL_4_BIT_SAMPLE_CLK_1_1 => '0',
	--	SIGNAL_4_BIT_1_2 => (others => '0'),
	--	SIGNAL_4_BIT_SAMPLE_CLK_1_2 => '0',
	--	SIGNAL_4_BIT_1_3 => (others => '0'),
	--	SIGNAL_4_BIT_SAMPLE_CLK_1_3 => '0',
	--	SIGNAL_4_BIT_1_4 => (others => '0'),
	--	SIGNAL_4_BIT_SAMPLE_CLK_1_4 => '0',
		SIGNAL_8_BIT_1_1 => S1_1,
		SIGNAL_8_BIT_SAMPLE_CLK_1_1 => S1_1B_CLK,
		SIGNAL_8_BIT_1_2 => S1_2,
		SIGNAL_8_BIT_SAMPLE_CLK_1_2 => S1_2B_CLK,
		SIGNAL_8_BIT_1_3 => S1_3,
		SIGNAL_8_BIT_SAMPLE_CLK_1_3 => S1_3B_CLK,
		SIGNAL_8_BIT_1_4 => S1_4,
		SIGNAL_8_BIT_SAMPLE_CLK_1_4 => S1_4B_CLK,
	--	SIGNAL_16_BIT_1_1 => (others => '0'),
	--	SIGNAL_16_BIT_SAMPLE_CLK_1_1 => '0',
	--	SIGNAL_16_BIT_1_2 => (others => '0'),
	--	SIGNAL_16_BIT_SAMPLE_CLK_1_2 => '0',
	--	SIGNAL_16_BIT_1_3 => (others => '0'),
	--	SIGNAL_16_BIT_SAMPLE_CLK_1_3 => '0',
	--	SIGNAL_16_BIT_1_4 => (others => '0'),
	--	SIGNAL_16_BIT_SAMPLE_CLK_1_4 => '0',
		SIGNAL_1_BIT_2_1 => '0',
		SIGNAL_1_BIT_SAMPLE_CLK_2_1 => '0',
		SIGNAL_1_BIT_2_2 => '0',
		SIGNAL_1_BIT_SAMPLE_CLK_2_2 => '0',
		SIGNAL_1_BIT_2_3 => '0',
		SIGNAL_1_BIT_SAMPLE_CLK_2_3 => '0',
		SIGNAL_1_BIT_2_4 => '0',
		SIGNAL_1_BIT_SAMPLE_CLK_2_4 => '0',
	--	SIGNAL_2_BIT_2_1 => (others => '0'),
	--	SIGNAL_2_BIT_SAMPLE_CLK_2_1 => '0',
	--	SIGNAL_2_BIT_2_2 => (others => '0'),
	--	SIGNAL_2_BIT_SAMPLE_CLK_2_2 => '0',
	--	SIGNAL_2_BIT_2_3 => (others => '0'),
	--	SIGNAL_2_BIT_SAMPLE_CLK_2_3 => '0',
	--	SIGNAL_2_BIT_2_4 => (others => '0'),
	--	SIGNAL_2_BIT_SAMPLE_CLK_2_4 => '0',
	--	SIGNAL_4_BIT_2_1 => (others => '0'),
	--	SIGNAL_4_BIT_SAMPLE_CLK_2_1 => '0',
	--	SIGNAL_4_BIT_2_2 => (others => '0'),
	--	SIGNAL_4_BIT_SAMPLE_CLK_2_2 => '0',
	--	SIGNAL_4_BIT_2_3 => (others => '0'),
	--	SIGNAL_4_BIT_SAMPLE_CLK_2_3 => '0',
	--	SIGNAL_4_BIT_2_4 => (others => '0'),
	--	SIGNAL_4_BIT_SAMPLE_CLK_2_4 => '0',
		SIGNAL_8_BIT_2_1 => S2_1,
		SIGNAL_8_BIT_SAMPLE_CLK_2_1 => S2_1B_CLK,
		SIGNAL_8_BIT_2_2 => S2_2,
		SIGNAL_8_BIT_SAMPLE_CLK_2_2 => S2_2B_CLK,
		SIGNAL_8_BIT_2_3 => S2_3,
		SIGNAL_8_BIT_SAMPLE_CLK_2_3 => S2_3B_CLK,
		SIGNAL_8_BIT_2_4 => S2_4,
		SIGNAL_8_BIT_SAMPLE_CLK_2_4 => S2_4B_CLK,
	--	SIGNAL_16_BIT_2_1 => (others => '0'),
	--	SIGNAL_16_BIT_SAMPLE_CLK_2_1 => '0',
	--	SIGNAL_16_BIT_2_2 => (others => '0'),
	--	SIGNAL_16_BIT_SAMPLE_CLK_2_2 => '0',
	--	SIGNAL_16_BIT_2_3 => (others => '0'),
	--	SIGNAL_16_BIT_SAMPLE_CLK_2_3 => '0',
	--	SIGNAL_16_BIT_2_4 => (others => '0'),
	--	SIGNAL_16_BIT_SAMPLE_CLK_2_4 => '0',
		SIGNAL_1_BIT_3_1 => '0',
		SIGNAL_1_BIT_SAMPLE_CLK_3_1 => '0',
		SIGNAL_1_BIT_3_2 => '0',
		SIGNAL_1_BIT_SAMPLE_CLK_3_2 => '0',
		SIGNAL_1_BIT_3_3 => '0',
		SIGNAL_1_BIT_SAMPLE_CLK_3_3 => '0',
		SIGNAL_1_BIT_3_4 => '0',
		SIGNAL_1_BIT_SAMPLE_CLK_3_4 => '0',
	--	SIGNAL_2_BIT_3_1 => (others => '0'),
	--	SIGNAL_2_BIT_SAMPLE_CLK_3_1 => '0',
	--	SIGNAL_2_BIT_3_2 => (others => '0'),
	--	SIGNAL_2_BIT_SAMPLE_CLK_3_2 => '0',
	--	SIGNAL_2_BIT_3_3 => (others => '0'),
	--	SIGNAL_2_BIT_SAMPLE_CLK_3_3 => '0',
	--	SIGNAL_2_BIT_3_4 => (others => '0'),
	--	SIGNAL_2_BIT_SAMPLE_CLK_3_4 => '0',
	--	SIGNAL_4_BIT_3_1 => (others => '0'),
	--	SIGNAL_4_BIT_SAMPLE_CLK_3_1 => '0',
	--	SIGNAL_4_BIT_3_2 => (others => '0'),
	--	SIGNAL_4_BIT_SAMPLE_CLK_3_2 => '0',
	--	SIGNAL_4_BIT_3_3 => (others => '0'),
	--	SIGNAL_4_BIT_SAMPLE_CLK_3_3 => '0',
	--	SIGNAL_4_BIT_3_4 => (others => '0'),
	--	SIGNAL_4_BIT_SAMPLE_CLK_3_4 => '0',
		SIGNAL_8_BIT_3_1 => S3_1,
		SIGNAL_8_BIT_SAMPLE_CLK_3_1 => S3_1B_CLK,
		SIGNAL_8_BIT_3_2 => S3_2,
		SIGNAL_8_BIT_SAMPLE_CLK_3_2 => S3_2B_CLK,
		SIGNAL_8_BIT_3_3 => S3_3,
		SIGNAL_8_BIT_SAMPLE_CLK_3_3 => S3_3B_CLK,
		SIGNAL_8_BIT_3_4 => (others => '0'),
		SIGNAL_8_BIT_SAMPLE_CLK_3_4 => '0',
	--	SIGNAL_16_BIT_3_1 => (others => '0'),
	--	SIGNAL_16_BIT_SAMPLE_CLK_3_1 => '0',
	--	SIGNAL_16_BIT_3_2 => (others => '0'),
	--	SIGNAL_16_BIT_SAMPLE_CLK_3_2 => '0',
	--	SIGNAL_16_BIT_3_3 => (others => '0'),
	--	SIGNAL_16_BIT_SAMPLE_CLK_3_3 => '0',
	--	SIGNAL_16_BIT_3_4 => (others => '0'),
	--	SIGNAL_16_BIT_SAMPLE_CLK_3_4 => '0',
		SIGNAL_1_BIT_4_1 => '0',
		SIGNAL_1_BIT_SAMPLE_CLK_4_1 => '0',
		SIGNAL_1_BIT_4_2 => '0',
		SIGNAL_1_BIT_SAMPLE_CLK_4_2 => '0',
		SIGNAL_1_BIT_4_3 => '0',
		SIGNAL_1_BIT_SAMPLE_CLK_4_3 => '0',
		SIGNAL_1_BIT_4_4 => '0',
		SIGNAL_1_BIT_SAMPLE_CLK_4_4 => '0',
	--	SIGNAL_2_BIT_4_1 => (others => '0'),
	--	SIGNAL_2_BIT_SAMPLE_CLK_4_1 => '0',
	--	SIGNAL_2_BIT_4_2 => (others => '0'),
	--	SIGNAL_2_BIT_SAMPLE_CLK_4_2 => '0',
	--	SIGNAL_2_BIT_4_3 => (others => '0'),
	--	SIGNAL_2_BIT_SAMPLE_CLK_4_3 => '0',
	--	SIGNAL_2_BIT_4_4 => (others => '0'),
	--	SIGNAL_2_BIT_SAMPLE_CLK_4_4 => '0',
	--	SIGNAL_4_BIT_4_1 => (others => '0'),
	--	SIGNAL_4_BIT_SAMPLE_CLK_4_1 => '0',
	--	SIGNAL_4_BIT_4_2 => (others => '0'),
	--	SIGNAL_4_BIT_SAMPLE_CLK_4_2 => '0',
	--	SIGNAL_4_BIT_4_3 => (others => '0'),
	--	SIGNAL_4_BIT_SAMPLE_CLK_4_3 => '0',
	--	SIGNAL_4_BIT_4_4 => (others => '0'),
	--	SIGNAL_4_BIT_SAMPLE_CLK_4_4 => '0',
		SIGNAL_8_BIT_4_1 => (others => '0'),
		SIGNAL_8_BIT_SAMPLE_CLK_4_1 => '0',
		SIGNAL_8_BIT_4_2 => (others => '0'),
		SIGNAL_8_BIT_SAMPLE_CLK_4_2 => '0',
		SIGNAL_8_BIT_4_3 => (others => '0'),
		SIGNAL_8_BIT_SAMPLE_CLK_4_3 => '0',
		SIGNAL_8_BIT_4_4 => (others => '0'),
		SIGNAL_8_BIT_SAMPLE_CLK_4_4 => '0',
	--	SIGNAL_16_BIT_4_1 => (others => '0'),
	--	SIGNAL_16_BIT_SAMPLE_CLK_4_1 => '0',
	--	SIGNAL_16_BIT_4_2 => (others => '0'),
	--	SIGNAL_16_BIT_SAMPLE_CLK_4_2 => '0',
	--	SIGNAL_16_BIT_4_3 => (others => '0'),
	--	SIGNAL_16_BIT_SAMPLE_CLK_4_3 => '0',
	--	SIGNAL_16_BIT_4_4 => (others => '0'),
	--	SIGNAL_16_BIT_SAMPLE_CLK_4_4 => '0',
		TRIGGER_1_BIT_1 => T1,
		TRIGGER_1_BIT_SAMPLE_CLK_1 => T1_CLK,
		TRIGGER_1_BIT_2 => T2,
		TRIGGER_1_BIT_SAMPLE_CLK_2 => T2_CLK,
		TRIGGER_1_BIT_3 => '0',
		TRIGGER_1_BIT_SAMPLE_CLK_3 => '0',
		TRIGGER_1_BIT_4 => '0',
		TRIGGER_1_BIT_SAMPLE_CLK_4 => '0',
	--	TRIGGER_2_BIT_1 => (others => '0'),
	--	TRIGGER_2_BIT_SAMPLE_CLK_1 => '0',
	--	TRIGGER_2_BIT_2 => (others => '0'),
	--	TRIGGER_2_BIT_SAMPLE_CLK_2 => '0',
	--	TRIGGER_2_BIT_3 => (others => '0'),
	--	TRIGGER_2_BIT_SAMPLE_CLK_3 => '0',
	--	TRIGGER_2_BIT_4 => (others => '0'),
	--	TRIGGER_2_BIT_SAMPLE_CLK_4 => '0',
	--	TRIGGER_4_BIT_1 => (others => '0'),
	--	TRIGGER_4_BIT_SAMPLE_CLK_1 => '0',
	--	TRIGGER_4_BIT_2 => (others => '0'),
	--	TRIGGER_4_BIT_SAMPLE_CLK_2 => '0',
	--	TRIGGER_4_BIT_3 => (others => '0'),
	--	TRIGGER_4_BIT_SAMPLE_CLK_3 => '0',
	--	TRIGGER_4_BIT_4 => (others => '0'),
	--	TRIGGER_4_BIT_SAMPLE_CLK_4 => '0',
		TRIGGER_8_BIT_1 => (others => '0'),
		TRIGGER_8_BIT_SAMPLE_CLK_1 => '0',
		TRIGGER_8_BIT_2 => (others => '0'),
		TRIGGER_8_BIT_SAMPLE_CLK_2 => '0',
		TRIGGER_8_BIT_3 => (others => '0'),
		TRIGGER_8_BIT_SAMPLE_CLK_3 => '0',
		TRIGGER_8_BIT_4 => (others => '0'),
		TRIGGER_8_BIT_SAMPLE_CLK_4 => '0',
	--	TRIGGER_16_BIT_1 => (others => '0'),
	--	TRIGGER_16_BIT_SAMPLE_CLK_1 => '0',
	--	TRIGGER_16_BIT_2 => (others => '0'),
	--	TRIGGER_16_BIT_SAMPLE_CLK_2 => '0',
	--	TRIGGER_16_BIT_3 => (others => '0'),
	--	TRIGGER_16_BIT_SAMPLE_CLK_3 => '0',
	--	TRIGGER_16_BIT_4 => (others => '0'),
	--	TRIGGER_16_BIT_SAMPLE_CLK_4 => '0',
		REG250 => REG250,
		REG251 => REG251,
		TRIGGER_REARM_TOGGLE => TRIGGER_REARM_TOGGLE,
		FORCE_TRIGGER_TOGGLE => FORCE_TRIGGER_TOGGLE,
		SREG250_READ_TOGGLE => SREG250_READ_TOGGLE,
		START_CAPTURE_TOGGLE => START_CAPTURE_TOGGLE,
		TP => open
	);
end generate;

--// TEST POINTS --------------------------------------------
TPS_OUT: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		RIGHT_CONNECTOR_A(10 downto 1) <= COM5402_TP;
--		RIGHT_CONNECTOR_A(1) <= LAN1_TCP_RX_DATA_VALID(0);
--		RIGHT_CONNECTOR_A(2) <= LAN1_TCP_RX_CTS;
--		RIGHT_CONNECTOR_A(3) <= RX_LAN_SAMPLE_CLK_REQ;
--		RIGHT_CONNECTOR_A(4) <= RX_LAN_BUFFER_EMPTY;
--		RIGHT_CONNECTOR_A(5) <= TX_LAN_TOGGLE;
		RIGHT_CONNECTOR_A(11) <= LAN1_TCP_TX_DATA_VALID(0);
--		RIGHT_CONNECTOR_A(7) <= LAN1_TCP_TX_CTS(0);
--		RIGHT_CONNECTOR_A(8) <= LAN1_MAC_TX_DATA_VALID;
--		RIGHT_CONNECTOR_A(9) <= LAN1_MAC_TX_CTS;
--		RIGHT_CONNECTOR_A(10) <= LAN1_MAC_RX_DATA_VALID;
--		RIGHT_CONNECTOR_A(11) <= TX_LAN_TOGGLE;
--		RIGHT_CONNECTOR_A(1) <= 
--		RIGHT_CONNECTOR_A(2) <= 
--		RIGHT_CONNECTOR_A(3) <= 
--		RIGHT_CONNECTOR_A(4) <= 
	end if;
end process;



--// SIM2OUTFILE --------------------------------------------
-- simulation. send to a tab-delimited text file for subsequent (Matlab) plot.

--SIM2OUTFILE_X: if(SIMULATION = '1') generate
--	SIM2OUTFILE_001: SIM2OUTFILE 
--	generic map(
--		FILENAME => "top1.txt"
--	)
--	PORT MAP(
--		CLK => CLK_P,
--		DATA1 => DATA1I(13 downto 2),	-- receiver1 output I,
--		DATA2 => DATA1Q(13 downto 2),	-- receiver1 output Q,
--		DATA3 => x"000",
--		SAMPLE_CLK_IN => '1'
--	);
--
--	SIM2OUTFILE_002: SIM2OUTFILE 
--	generic map(
--		FILENAME => "top2.txt"
--	)
--	PORT MAP(
--		CLK => CLK_DACg,
--		DATA1 => MOD_DATA_I(17 downto 6),	-- 
--		DATA2 => MOD_DATA_Q(17 downto 6),
--		DATA3 => x"000",
--		SAMPLE_CLK_IN => '1'
--	);
--
--
--
--end generate;

--// ARM CO-PROCESSOR INTERFACE ---------------------------------------------------
-- Primarily used for monitoring and control.
-- 8-bit shared address/data bus. clock synchronous.
-- PART OF THE FRAMEWORK. DO NOT CHANGE unless you want to 
-- add more control registers (see UC_WRITE_001) or status registers (see UC_READ_001).
-- 
-- Address latch
UC_ADDR_001: process(UC_CSIB_IN, UC_WEBG, UC_ALE_IN, UC_AD)
begin
	if rising_edge(UC_WEBG) then
		if (UC_CSIB_IN = '0') and (UC_ALE_IN = '1') then
			UC_ADDRESS <= UC_AD;
	  	end if;
  	end if;
end process;

-- Read status registers 
UC_READ_001: process(UC_ADDRESS, REG255, RX_USB_LAN, REG251, REG250, SREG6, SREG7, 
	SREG10, SREG11, SREG12, SREG13, SREG14, SREG15, SREG16, SREG17, SREG18, SREG19, 
	SREG20, SREG21, SREG22, SREG23, SREG24, SREG25, SREG26, SREG27, SREG28)
begin
	case(UC_ADDRESS) is
		-- ComBlock-reserved addresses ----
		when "11111111" => UC_AD_LOCAL <= REG255;		-- reg 255. Async serial from 3 connectors
		when "11111110" => UC_AD_LOCAL <= RX_USB_LAN;-- reg 254 = 8-bit data from USB or LAN M&C channel
		when "11111101" => UC_AD_LOCAL <= REVISION;	-- reg 253 = fpga revision
		when "11111100" => UC_AD_LOCAL <= OPTION;		-- reg 252 = fpga option
		when "11111011" => UC_AD_LOCAL <= REG251;		-- reg 251 =comscope  status
		when "11111010" => UC_AD_LOCAL <= REG250;		-- reg 250 = comscope data byte
		
		-- User-defined status registers in microcontroller -------
      -- reserved SREG0-SREG7
		when "00000110" => UC_AD_LOCAL <= SREG6;	-- USB PHY ID
		when "00000111" => UC_AD_LOCAL <= SREG7;	-- GbE LAN PHY ID
		
		-- User-defined status registers -------
--		when "00001000" => UC_AD_LOCAL <= SREG8;	
		when "00001001" => UC_AD_LOCAL <= SREG9;	
		when "00001010" => UC_AD_LOCAL <= SREG10;	
		when "00001011" => UC_AD_LOCAL <= SREG11;
		when "00001100" => UC_AD_LOCAL <= SREG12;
		when "00001101" => UC_AD_LOCAL <= SREG13;
		when "00001110" => UC_AD_LOCAL <= SREG14;
		when "00001111" => UC_AD_LOCAL <= SREG15;
		when "00010000" => UC_AD_LOCAL <= SREG16;
		when "00010001" => UC_AD_LOCAL <= SREG17;
		when "00010010" => UC_AD_LOCAL <= SREG18;
		when "00010011" => UC_AD_LOCAL <= SREG19;
		when "00010100" => UC_AD_LOCAL <= SREG20;
		when "00010101" => UC_AD_LOCAL <= SREG21;
		when "00010110" => UC_AD_LOCAL <= SREG22;
		when "00010111" => UC_AD_LOCAL <= SREG23;
		when "00011000" => UC_AD_LOCAL <= SREG24;
		when "00011001" => UC_AD_LOCAL <= SREG25;
		when "00011010" => UC_AD_LOCAL <= SREG26;
		when "00011011" => UC_AD_LOCAL <= SREG27;
		when "00011100" => UC_AD_LOCAL <= SREG28;
		
		-- user can add/remove status registers as needed (200+)
		when others => UC_AD_LOCAL <= (others => '0');
	end case;
end process;

UC_AD <= UC_AD_LOCAL when (UC_CSIB_IN = '0') and (UC_REB_IN = '0') else (others => 'Z');
	-- when to drive the data bus between ARM and FPGA
	
-- toggle a few flags upon reading specific status registers
UC_READ_002: process(ASYNC_RESET, UC_REBG, UC_CSIB_IN, UC_ADDRESS)
begin
	if(ASYNC_RESET = '1') then
		SREG8_READ_TOGGLE <= '0';
		SREG250_READ_TOGGLE <= '0';
		SREG254_READ_TOGGLE <= '0';
	elsif rising_edge(UC_REBG) then
		if (UC_CSIB_IN = '0')  then
		
			if (UC_ADDRESS = 8) then
				SREG8_READ_TOGGLE <= not SREG8_READ_TOGGLE;
			end if;
			if (UC_ADDRESS = 250) then
				SREG250_READ_TOGGLE <= not SREG250_READ_TOGGLE;
			end if;
			if (UC_ADDRESS = 254) then
				SREG254_READ_TOGGLE <= not SREG254_READ_TOGGLE;
			end if;
			
			
		end if;
	end if;
end process;

-- reclock with CLK_P
RECLOCK_SREG8_TOGGLE: process(CLK_P,SREG8_READ_TOGGLE)
begin
	if rising_edge(CLK_P) then
		SREG8_READ_TOGGLE_D <= SREG8_READ_TOGGLE;
		SREG8_READ_TOGGLE_D2 <= SREG8_READ_TOGGLE_D;
		if(SREG8_READ_TOGGLE_D /= SREG8_READ_TOGGLE_D2) then
			LATCH_MONITORING_PULSE <= '1';
		else
			LATCH_MONITORING_PULSE <= '0';
		end if;
	end if;
end process;
		
	
-- write to FPGA control registers
UC_WRITE_001: process(UC_CSIB_IN, UC_WEBG, UC_ALE_IN, UC_REB_IN, UC_AD)
begin
	if rising_edge(UC_WEBG) then
		if (UC_CSIB_IN = '0') and (UC_ALE_IN = '0') and (UC_REB_IN = '1')  then
			case UC_ADDRESS is
				-- registers REG0-6 reserved for ARM processor
				when x"07" => REG7 <= UC_AD;
				when x"08" => REG8 <= UC_AD;
				when x"09" => REG9 <= UC_AD;
				when x"0A" => REG10 <= UC_AD;
				when x"0B" => REG11 <= UC_AD;
				when x"0C" => REG12 <= UC_AD;
				when x"0D" => REG13 <= UC_AD;
				when x"0E" => REG14 <= UC_AD;
				when x"0F" => REG15 <= UC_AD;

				when x"10" => REG16 <= UC_AD;
				when x"11" => REG17 <= UC_AD;
				when x"12" => REG18 <= UC_AD;
				when x"13" => REG19 <= UC_AD;
				when x"14" => REG20 <= UC_AD;
				when x"15" => REG21 <= UC_AD;
				when x"16" => REG22 <= UC_AD;
				when x"17" => REG23 <= UC_AD;
				when x"18" => REG24 <= UC_AD;
				when x"19" => REG25 <= UC_AD;
				when x"1A" => REG26 <= UC_AD;
				when x"1B" => REG27 <= UC_AD;
				when x"1C" => REG28 <= UC_AD;
				when x"1D" => REG29 <= UC_AD;
				when x"1E" => REG30 <= UC_AD;
				when x"1F" => REG31 <= UC_AD;

				when others => null;
			end case;

			if (UC_ADDRESS = 31) then
				CONFIG_CHANGE_TOGGLE <= not CONFIG_CHANGE_TOGGLE;	
				-- Configuration change toggles upon writing the last control register
				-- At power up the ARM uC AUTOMATICALLY writes all control registers, including REG31.
			end if;


			-- REG254 reserved for output serial
			-- REG255 write reserved for output serial (8-bit parallel format)
			if (UC_ADDRESS = 255) then
				-- REG255 write reserved for output serial (8-bit parallel format)
				if(PATH_TO_HOST = 1) then
					-- route response to host query over USB
					TX_USB <= UC_AD;
					TX_USB_TOGGLE <= not TX_USB_TOGGLE;
				end if;
				if(PATH_TO_HOST = 2) then
					-- route response to host query over LAN/TCP-IP
					LAN1_TCP_TX_DATA(0) <= UC_AD;
					TX_LAN_TOGGLE <= not TX_LAN_TOGGLE;
				end if;
			end if;
		end if;
	end if;
end process;

--// reserved COMSCOPE -------------------------------------------------------
-- Comment out if ComScope feature is unused
UC_WRITE_003: process(UC_CSIB_IN, UC_WEBG, UC_ADDRESS, UC_AD, 
	FORCE_TRIGGER_TOGGLE, TRIGGER_REARM_TOGGLE, START_CAPTURE_TOGGLE)
begin
	if rising_edge(UC_WEBG) then
		if (UC_CSIB_IN = '0') and (UC_ALE_IN = '0') and (UC_REB_IN = '1')  then
			-- Reserved for ComScope			
			if (UC_ADDRESS = 237) then
				REG237 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 238) then
				REG238 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 239) then
				REG239 <= UC_AD;
			end if;
			if (UC_ADDRESS = 240) then
				REG240 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 241) then
				REG241 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 242) then
				REG242 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 243) then
				REG243 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 244) then
				REG244 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 245) then
				REG245 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 246) then
				REG246 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 247) then
				REG247 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 248) then
				REG248 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 249) then
				REG249 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 250) then 
				-- detect one-shot force trigger	
				if(UC_AD(0) = '1') then	
					FORCE_TRIGGER_TOGGLE <= not(FORCE_TRIGGER_TOGGLE);
				end if;	
				-- detect one-shot re-arm	
				if(UC_AD(1) = '1') then	
					TRIGGER_REARM_TOGGLE <= not(TRIGGER_REARM_TOGGLE);
				end if;	
				-- start capturing again
				if(UC_AD(2) = '1') then	
					START_CAPTURE_TOGGLE <= not (START_CAPTURE_TOGGLE);
				end if;	
			end if;
		end if;
	end if;
end process;

--// end of COMSCOPE section -------------------------------------------------------

-- Reclock CONFIG_CHANGE_TOGGLE to be synchronous with processing clock CLK_P
RECLOCK_CLK_P_001: process(CLK_P, CONFIG_CHANGE_TOGGLE)
begin
	if rising_edge(CLK_P) then
		CONFIG_CHANGE_TOGGLE_D <= CONFIG_CHANGE_TOGGLE;
		CONFIG_CHANGE_TOGGLE_D2 <= CONFIG_CHANGE_TOGGLE_D;
		if(CONFIG_CHANGE_TOGGLE_D /= CONFIG_CHANGE_TOGGLE_D2) then
			CONFIG_CHANGE_PULSE <= '1';
		else
			CONFIG_CHANGE_PULSE <= '0';
		end if;
	end if;
end process;

-- Reclock CONFIG_CHANGE_TOGGLE to be synchronous with processing clock CLK_DACg
RECLOCK_CLK_DACg_001: process(CLK_DACg, CONFIG_CHANGE_TOGGLE)
begin
	if rising_edge(CLK_DACg) then
		CONFIG_CHANGE_TOGGLE2_D <= CONFIG_CHANGE_TOGGLE;
		CONFIG_CHANGE_TOGGLE2_D2 <= CONFIG_CHANGE_TOGGLE2_D;
		if(CONFIG_CHANGE_TOGGLE2_D /= CONFIG_CHANGE_TOGGLE2_D2) then
			CONFIG_CHANGE_PULSE2 <= '1';
		else
			CONFIG_CHANGE_PULSE2 <= '0';
		end if;
	end if;
end process;

-- DNA Port ------------------------------------
-- Read FPGA unique DNA ID
DNA_ID_001: DNA_ID PORT MAP(
	CLK => CLK_P,
	ASYNC_RESET => ASYNC_RESET,
	ID_OUT => DNA_ID_DATA,
	ID_VALID_OUT => open,
	ID_SAMPLE_CLK_OUT => DNA_ID_SAMPLE_CLK 
);

-- M&C USB interface --------------------------
-- REG255 reserved for reading serial port inputs
-- These signals are pulled high (to mimic stop bit).
REG255 <= "1111" & (RX_USB_BUFFER_EMPTY and RX_LAN_BUFFER_EMPTY) & "111";
	-- M&C port shared between LAN and USB  
	-- indicates AVAILABILITY of 8-bit byte from either USB or LAN. Not actual data.
	-- When low, uC must read byte from USB or LAN


-- Reclock TX_USB_TOGGLE
RECLOCK_TX_USB_TOGGLE: process(CLK_P,TX_USB_TOGGLE)
begin
	if rising_edge(CLK_P) then
		TX_USB_TOGGLE_D <= TX_USB_TOGGLE;
		TX_USB_TOGGLE_D2 <= TX_USB_TOGGLE_D;
		if(TX_USB_TOGGLE_D /= TX_USB_TOGGLE_D2) then
			TX_USB_SAMPLE_CLK <= '1';
		else
			TX_USB_SAMPLE_CLK <= '0';
		end if;
	end if;
end process;

-- Reclock TX_LAN_TOGGLE
RECLOCK_TX_LAN_TOGGLE: process(CLK_P,TX_LAN_TOGGLE)
begin
	if rising_edge(CLK_P) then
		TX_LAN_TOGGLE_D <= TX_LAN_TOGGLE;
		TX_LAN_TOGGLE_D2 <= TX_LAN_TOGGLE_D;
		if(TX_LAN_TOGGLE_D /= TX_LAN_TOGGLE_D2) then
			LAN1_TCP_TX_DATA_VALID(0) <= '1';
		else
			LAN1_TCP_TX_DATA_VALID(0) <= '0';
		end if;
	end if;
end process;

-- Immediately upon completing reading a M&C byte from the USB or LAN, ask for the 
-- next one. 
-- RACE CONDITION??????? ARM IS FAST!!!!
RECLOCK_RDN_D: process(CLK_P, SREG254_READ_TOGGLE)
begin
	if rising_edge(CLK_P) then
		SREG254_READ_TOGGLE_D <= SREG254_READ_TOGGLE;
		SREG254_READ_TOGGLE_D2 <= SREG254_READ_TOGGLE_D;

		RX_USB_BUFFER_EMPTY_D <= RX_USB_BUFFER_EMPTY;
		RX_USB_BUFFER_EMPTY_D2 <= RX_USB_BUFFER_EMPTY_D;
		RX_LAN_BUFFER_EMPTY_D <= RX_LAN_BUFFER_EMPTY;
		RX_LAN_BUFFER_EMPTY_D2 <= RX_LAN_BUFFER_EMPTY_D;


		if (SREG254_READ_TOGGLE_D2 /= SREG254_READ_TOGGLE_D) then
			-- end of M&C byte read from ARM uC.
			if(RX_LAN_BUFFER_EMPTY_D2 = '0') then
				-- last M&C byte read was over LAN. Increment LAN elastic buffer read pointer
				RX_LAN_SAMPLE_CLK_REQ <= '1';
				RX_USB_SAMPLE_CLK_REQ <= '0';
				PATH_TO_HOST <= 2;	
					-- identifies LAN as the media through which	this module exchanges 
					-- M&C information. Response will be directed the same way.
			elsif(RX_USB_BUFFER_EMPTY_D2 = '0') then
				-- last M&C byte read was over USB. Increment USB elastic buffer read pointer
				RX_LAN_SAMPLE_CLK_REQ <= '0';
				RX_USB_SAMPLE_CLK_REQ <= '1';
				PATH_TO_HOST <= 1;	
					-- identifies USB as the media through which	this module exchanges 
					-- M&C information. Response will be directed the same way.
			end if;
		else
			RX_LAN_SAMPLE_CLK_REQ <= '0';
			RX_USB_SAMPLE_CLK_REQ <= '0';
		end if;
	end if;
end process;

-- read byte received over USB or LAN
RX_USB_LAN <= RX_USB when (RX_USB_BUFFER_EMPTY = '0') else RX_LAN;


end Behavioral;
