-------------------------------------------------------------
-- MSS copyright 2013
--	Filename:  XILINX_DCM_CLKGEN_PROG.VHD
-- Authors: 
--		Alain Zarembowitch / MSS, 
--	Version: 1
--	Date last modified: 5/11/13
-- Inheritance: 	n/a
--
-- description: Generate a clean programmable clock using the programmable port of DCM_CLKGEN 
-- primitive. Xilinx-specific.
-- 
-- Reference document: ug382
-- Usage: output clock frequency is CLK_IN_FREQ MHz * M / D, where M and D are programmable integers.
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
library unisim;
use unisim.vcomponents.all;

entity XILINX_DCM_CLKGEN_PROG is
generic (
   CLKIN_PERIOD: real := 10.0;
		-- Input clock period specified in nS
   CLKFX_MD_MAX: real := 2.0    
		-- Specify maximum M/D ratio for timing anlysis
);	
port
 (
  CLKIN: in     std_logic;
	-- input reference clock. Must be global.
	-- Minimum input frequency 5 MHz
  M_MINUS1: in std_logic_vector(7 downto 0);
	-- 1 <= (M-1) <= 255
	-- specifies the frequency multiplication factor M.
  D_MINUS1: in std_logic_vector(7 downto 0);
	-- 0 <= (D-1) <= 255
 	-- specifies the frequency division factor D
  CONFIG_CHANGE_TOGGLE: in std_logic;
	-- toggle to trigger a frequency change. 
	-- (re_programming starts at the falling edge of this pulse.
	-- The frequencies can be changed on the fly at run time. 
  CLKOUTG          : out    std_logic;
	-- Clock out ports (after BUFGs)
	-- 5 - 375 MHz (Spartan-6 -3 grade)
	-- 5 - 333 MHz (Spartan-6 -2 grade)
	-- 5 - 200 MHz (Spartan-6 -1L grade)
  READY            : out    std_logic := '0'
	-- '1' when the output clock becomes usable. A combination of LOCK and PROGDONE
 );
end entity;

architecture Behavioral of XILINX_DCM_CLKGEN_PROG is
--------------------------------------------------------
--      COMPONENTS
--------------------------------------------------------
--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------
signal CLKFX: std_logic := '0';
signal CONFIG_CHANGE_TOGGLE_D: std_logic := '0';
signal CONFIG_CHANGE_TOGGLE_D2: std_logic := '0';
signal CHANGE_CONFIG_TRIGGER: std_logic := '0';
signal STATE: integer range 0 to 5 := 0;
signal CNTR: integer range 0 to 10 := 0;
signal PROGDATA_REG: std_logic_vector(9 downto 0) := (others => '0');
signal PROGCLK: std_logic := '0';
signal PROGDATA: std_logic := '0';
signal PROGEN: std_logic := '0';
signal LOCKED: std_logic := '0';
signal PROGDONE: std_logic := '0';
--------------------------------------------------------
--      IMPLEMENTATION
--------------------------------------------------------
begin

   DCM_CLKGEN_inst : DCM_CLKGEN
   generic map (
      CLKFXDV_DIVIDE => 2,       -- CLKFXDV divide value (2, 4, 8, 16, 32)
      CLKFX_DIVIDE => 1,         -- Divide value - D - (1-256) 
      CLKFX_MD_MAX => CLKFX_MD_MAX,       -- Specify maximum M/D ratio for timing anlysis
      CLKFX_MULTIPLY => 2,       -- Multiply value - M - (2-256)
      CLKIN_PERIOD => CLKIN_PERIOD,       -- Input clock period specified in nS
      SPREAD_SPECTRUM => "NONE", -- Spread Spectrum mode "NONE", "CENTER_LOW_SPREAD", "CENTER_HIGH_SPREAD",
                                 -- "VIDEO_LINK_M0", "VIDEO_LINK_M1" or "VIDEO_LINK_M2" 
      STARTUP_WAIT => FALSE      -- Delay config DONE until DCM_CLKGEN LOCKED (TRUE/FALSE)
   )
   port map (
      CLKFX => CLKFX,         -- 1-bit output: Generated clock output
      CLKFX180 => open,   -- 1-bit output: Generated clock output 180 degree out of phase from CLKFX.
      CLKFXDV => open,     -- 1-bit output: Divided clock output
      LOCKED => LOCKED,       -- 1-bit output: Locked output
      PROGDONE => PROGDONE,   -- 1-bit output: Active high output to indicate the successful re-programming
      STATUS => open,       -- 2-bit output: DCM_CLKGEN status
      CLKIN => CLKIN,         -- 1-bit input: Input clock
      FREEZEDCM => '0', 		-- 1-bit input: Prevents frequency adjustments to input clock
      PROGCLK => PROGCLK,     -- 1-bit input: Clock input for M/D reconfiguration
      PROGDATA => PROGDATA,   -- 1-bit input: Serial data input for M/D reconfiguration
      PROGEN => PROGEN,       -- 1-bit input: Active high program enable
      RST => '0'              -- 1-bit input: Reset input pin
   );

-- global clock
BUFG_001: BUFG port map(I => CLKFX, O => CLKOUTG);

-- make programming trigger clock synchronous
SERIAL_PROG_001: process(CLKIN)
begin
	if rising_edge(CLKIN) then
		CONFIG_CHANGE_TOGGLE_D <= CONFIG_CHANGE_TOGGLE;
		CONFIG_CHANGE_TOGGLE_D2 <= CONFIG_CHANGE_TOGGLE_D;
		
		if(CONFIG_CHANGE_TOGGLE_D /= CONFIG_CHANGE_TOGGLE_D2) then
			-- frequency configuration change. start programming
			CHANGE_CONFIG_TRIGGER <= '1';
		else 
			CHANGE_CONFIG_TRIGGER <= '0';
		end if;
	end if;
end process;

-- serial programming state machine
-- See ug382 Figure 2-14
SERIAL_PROG_002: process(CLKIN)
begin
	if rising_edge(CLKIN) then
		if(STATE = 0) and (CHANGE_CONFIG_TRIGGER = '1') then
			-- frequency configuration change. LoadD
			STATE <= 1;
			CNTR <= 10;
			PROGEN <= '1';
			PROGDATA_REG <= "10" & D_MINUS1(0) & D_MINUS1(1) & D_MINUS1(2) & D_MINUS1(3) & D_MINUS1(4) & D_MINUS1(5) & D_MINUS1(6) & D_MINUS1(7);
		elsif(STATE = 1) and (CNTR = 1) then
			-- gap between LoadD and LoadM
			STATE <= 2;
			PROGEN <= '0';
			PROGDATA_REG(9) <= '0';
			CNTR <= 3;
		elsif(STATE = 2) and (CNTR = 1) then
			-- frequency configuration change. LoadM
			STATE <= 3;
			CNTR <= 10;
			PROGEN <= '1';
			PROGDATA_REG <= "11" & M_MINUS1(0) & M_MINUS1(1) & M_MINUS1(2) & M_MINUS1(3) & M_MINUS1(4) & M_MINUS1(5) & M_MINUS1(6) & M_MINUS1(7);
		elsif(STATE = 3) and (CNTR = 1) then
			-- gap between LoadM and Go command
			STATE <= 4;
			PROGEN <= '0';
			PROGDATA_REG(9) <= '0';
			CNTR <= 3;
		elsif(STATE = 4) and (CNTR = 1) then
			-- Go command
			STATE <= 5;
			PROGEN <= '1';
		elsif(STATE = 5) then
			-- programming is complete, back to idle
			STATE <= 0;
			PROGEN <= '0';
			CNTR <= 0;
		elsif(CNTR /= 0) then
			CNTR <= CNTR -1;
			if(STATE = 1) or (STATE = 3) then
				PROGDATA_REG(9 downto 1) <= PROGDATA_REG(8 downto 0);	-- shift left
			else
				PROGDATA_REG(9) <= '0';
			end if;
		end if;
	end if;
end process;

PROGCLK <= CLKIN;
PROGDATA <= PROGDATA_REG(9);

READY_GEN_001: process(CLKIN)
begin
	if rising_edge(CLKIN) then
		if(STATE /= 0) then
			READY <= '0';
		elsif(LOCKED = '1') then
			READY <= '1';
		end if;
	end if;
end process;



end Behavioral;
